/* board_ua1arn.c */

#include <stdlib.h>
#include <assert.h>

#include "board.h"
#include "product.h"
#include "synthcalcs.h"

/* Pin manipulation functions */
#include <avr/io.h>
#include <util/delay.h>
//#include <avr/pgmspace.h>
#include "spifuncs.h"


#if BOARD_SUPPORT_UA1ARN



#if defined (__AVR_ATmega128__)

#define MCU_TARGET_PORT PORTC

#elif defined (__AVR_ATmega32__) || defined (__AVR_ATmega16__)

#define MCU_TARGET_PORT PORTC

#endif

void
lpt_writeb(unsigned char c)
{
	MCU_TARGET_PORT = c;
}

void
lpt_and_writeb(unsigned char data, unsigned char mask)
{
	MCU_TARGET_PORT = (MCU_TARGET_PORT & ~ mask) | (data & mask);
}

void lpt_set_d0_0(void)
{
	MCU_TARGET_PORT &= ~0x01;
}

void lpt_set_d0_1(void)
{
	MCU_TARGET_PORT |= 0x01;
}

/*
void lpt_set_d0(uint8_t value)
{
	if (value)
		MCU_TARGET_PORT |= 0x01;
	else
		MCU_TARGET_PORT &= ~0x01;
}
*/

void lpt_set_d1(uint8_t value)
{
	if (value != 0)
		MCU_TARGET_PORT |= 0x02;
	else
		MCU_TARGET_PORT &= ~0x02;
}

/*
void lpt_set_d2(uint8_t value)
{
	if (value)
		MCU_TARGET_PORT |= 0x04;
	else
		MCU_TARGET_PORT &= ~0x04;
}

void lpt_set_d3(uint8_t value)
{
	if (value)
		MCU_TARGET_PORT |= 0x08;
	else
		MCU_TARGET_PORT &= ~0x08;
}

void lpt_set_d4(uint8_t value)
{
	if (value)
		MCU_TARGET_PORT |= 0x10;
	else
		MCU_TARGET_PORT &= ~0x10;
}

*/

void lpt_set_d5(uint8_t value)
{
	if (value)
		MCU_TARGET_PORT |= 0x20;
	else
		MCU_TARGET_PORT &= ~0x20;
}

void lpt_set_d6(uint8_t value)
{
	if (value)
		MCU_TARGET_PORT |= 0x40;
	else
		MCU_TARGET_PORT &= ~0x40;
}

void lpt_set_d7(uint8_t value)
{
	if (value)
		MCU_TARGET_PORT |= 0x80;
	else
		MCU_TARGET_PORT &= ~0x80;
}




/* make pulse on ioreset line */
void prog_pulse_ioreset(void)
{
	lpt_set_d6(1);
  _delay_us(1);               /* guarantee 1000 ns high */
	lpt_set_d6(0);
}


/* make pulse on ioupdate line */
void prog_pulse_ioupdate(void)
{
	lpt_set_d7(1);
  _delay_us(1);               /* guarantee 1000 ns high */
	lpt_set_d7(0);
}



void phase_fromulong(
	phase_t * o,
	unsigned long arg
	)
{
	* o = arg;
}
void phase_fromuint(
	phase_t * o,
	unsigned arg
	)
{
	* o = arg;
}

void prog_pulse_ioupdate(void);


#define PLL1_GO_STATE 1	/* N19=LOW ICPo = 250 A, N19=HIGH ICPo = 1 mA. */
#define PLL3_GO_STATE 1 /* N19=LOW ICPo = 250 A, N19=HIGH ICPo = 1 mA. */

/**********************/
#if defined(PLL1_TYPE)
#define PLLX_TYPE PLL1_TYPE
#elif defined(PLL2_TYPE)
#define PLLX_TYPE PLL2_TYPE
#endif

#if defined(AMP1_TYPE)
#define AMPX_TYPE AMP1_TYPE
#elif defined(AMP2_TYPE)
#define AMPX_TYPE AMP2_TYPE
#elif defined(AMP3_TYPE)
#define AMPX_TYPE AMP3_TYPE
#endif

#if (PLLX_TYPE == PLL_TYPE_ADF4001)
#include "chip\adf4001.c"
#elif (PLLX_TYPE == PLL_TYPE_LMX2306)
#include "chip\lmx2306.c"
#elif (PLLX_TYPE == PLL_TYPE_MC145170)
#include "chip\mc145170.c"
#endif

#if (DDS1_TYPE == DDS_TYPE_AD9852)
#include "chip\ad9852.c"
#elif (DDS1_TYPE == DDS_TYPE_AD9857)
#include "chip\ad9857.c"
#elif (DDS1_TYPE == DDS_TYPE_AD9951)
#include "chip\ad9951.c"
#elif (DDS1_TYPE == DDS_TYPE_AD9834)
#include "chip\ad9834.c"
#elif (DDS1_TYPE == DDS_TYPE_AD9851)
#include "chip\ad9851.c"
#elif (DDS2_TYPE == DDS_TYPE_AD9832)
#include "chip\ad9832.c"
#endif

#if (DDS2_TYPE == DDS_TYPE_AD9852)
#include "chip\ad9852.c"
#elif (DDS2_TYPE == DDS_TYPE_AD9857)
#include "chip\ad9857.c"
#elif (DDS2_TYPE == DDS_TYPE_AD9951)
#include "chip\ad9951.c"
#elif (DDS2_TYPE == DDS_TYPE_AD9834)
#include "chip\ad9834.c"
#elif (DDS2_TYPE == DDS_TYPE_AD9851)
#include "chip\ad9851.c"
#elif (DDS2_TYPE == DDS_TYPE_AD9832)
#include "chip\ad9832.c"
#endif

#if (AMPX_TYPE == AMP_TYPE_AD8320)
#include "chip\ad8320.c"
#elif (AMPX_TYPE == AMP_TYPE_AD8321)
#include "chip\ad8321.c"
#endif

/********************************/

static phase_t phase_last_n1;
static uint16_t board_vfos [3];
#define VFODIVPOWER 12			/* ~ 4 kHz granulation */

unsigned char iAmp1Level = 58; //0x64; //0x8e; //0x60; //0xff;
unsigned char iAmp2Level = 58; //0x64; //0x60; //0xff;
unsigned char iAmp3Level = 58; //0x64; //0x60; //0xff;

// for programming PLL2
static phase_t phase_lo2_r;
static phase_t phase_lo2_n;

const phase_t phase_0 = 0;


////////////////
// board specific functions

static unsigned char glob_agc_off;
static unsigned char glob_agc_code;
static unsigned char glob_ammode;
static unsigned char glob_filter;
static unsigned char glob_att;

static unsigned char glob_vco;
static unsigned char glob_loamp1;
static unsigned char glob_loamp2;
static unsigned char glob_loamp3;
static unsigned char glob_allamp;


static void prog_ctrlreg(void)
{
	static unsigned char vcos [4] =
	{
		0x01,	// vco 0
		0x02,	// vco 1
		0x04,	// vco 2
		0x08,	// vco 3
	};
	const unsigned char target = targetctl1;	/* addressing to chip */


	prog_select(target, 1);	/* start sending data to target chip */

	// programming RX control registers
	// filter codes:
	// 0x06 - fil3 3.1 kHz
	// 0x01 - fil2 2.7 kHz
	// 0x00 - fil0 9.0 kHz
	// 0x02 - fil1 6.0 kHz
	// 0x03 - fil5 0.5 kHz
	// 0x04 - fil2 15 kHz (bypass)
	// 0x05 - unused
	// 0x07 - unused

	prog_val(target, glob_filter, 3);	/* select IF filter, wrong order of bits */
	prog_bit(target, 0x00);		/* unused output */
	prog_bit(target, 0x00);		/* unused output */
	prog_bit(target, glob_ammode == 0x03);		/* FM DETECTOR POWER */
	prog_bit(target, glob_att);							/* ATTENUATOR RELAYS POWER */
	prog_bit(target, ! glob_att);						/* RF AMP POWER */

	prog_bit(target, glob_agc_off);		/* AGC OFF */
	prog_val(target, glob_agc_code, 3);	/* AGC code (delay) */
	prog_val(target, glob_ammode, 2);	/* AF input selection 0-ssb, 1-am, 3-fm */
	prog_bit(target, 0x00);		/* AF and IF OFF */
	prog_bit(target, glob_ammode == 0x03 || glob_ammode == 0x01);	// switch lo3 off in AM and FM modes		/* Unused output */


	prog_bit(target, glob_loamp2);		/* D7 ni control register */
	prog_bit(target, glob_loamp3);
	prog_bit(target, glob_loamp1);
	prog_bit(target, glob_allamp);		/* d4 in control register */
	prog_val(target, vcos [glob_vco], 4);	/* d0..d3 in control register */
	//prog_val(target, 0x0f, 4);	/* d0..d3 in control register */
	//prog_val(target, 0x01, 4);	/* d0..d3 in control register */

	prog_select(target, 0);	/* done sending data to target chip */

}


/* ctl register interface */



static void board_ctl_init()
{

	glob_agc_off =
	glob_agc_code =
	glob_ammode =
	glob_filter = 0x00;

	glob_vco = 0;
	glob_loamp1 =
	glob_loamp2 =
	glob_loamp3 =
	glob_allamp = 0x00;

	prog_ctrlreg(); 
}


void board_set_detector(unsigned char n)
{
	if (glob_ammode != n)
	{
		glob_ammode = n;
		prog_ctrlreg();
	}
}

void board_set_filter(unsigned char n)
{
	if (glob_filter != n)
	{
		glob_filter = n;
		prog_ctrlreg(); 
	}
}

void board_set_agc(unsigned char off, unsigned char code)
{
	if (glob_agc_off != off || glob_agc_code != code)
	{
		glob_agc_off = off;
		glob_agc_code = code;
		prog_ctrlreg(); 
	}
}

void board_set_att(unsigned char v)
{
	unsigned char n = v != 0;
	if (glob_att != n)
	{
		glob_att = n;
		prog_ctrlreg(); 
	}
}

static void board_ctl_set_vco(
	unsigned char n)	// 0..3 - code of VCO
{
	if (glob_vco != n)
	{
		glob_vco = n;
		prog_ctrlreg(); 
	}
}

void board_set_loamp1(unsigned char v)
{
	unsigned char n = v != 0;
	if (glob_loamp1 != n)
	{
		glob_loamp1 = n;
		prog_ctrlreg(); 
	}
}
void board_set_loamp2(unsigned char v)
{
	unsigned char n = v != 0;
	if (glob_loamp2 != n)
	{
		glob_loamp2 = n;
		prog_ctrlreg(); 
	}
}
void board_set_loamp3(unsigned char v)
{
	unsigned char n = v != 0;
	if (glob_loamp3 != n)
	{
		glob_loamp3 = n;
		prog_ctrlreg(); 
	}
}
void board_set_allamp(unsigned char v)
{
	unsigned char n = v != 0;
	if (glob_allamp != n)
	{
		glob_allamp = n;
		prog_ctrlreg(); 
	}
}

/*    -        */
void board_set_lsb(unsigned char v)	/*      */
{

}



static unsigned char getvfonumber(
	unsigned long mfreq // 1-st lo freq
	)
{
	const uint16_t mfreql = (uint16_t) (mfreq >> VFODIVPOWER);
	unsigned char i;

	for (i = 0; i < sizeof board_vfos / sizeof board_vfos [0]; ++ i)
	{
		if (board_vfos [i] > mfreql)
			break;
	}
	return i;
}


void prog_vfo_divider(long divider)
{

}

void prog_vfo_select(long freq)
{
	board_ctl_set_vco(getvfonumber(freq));	// optimizations already inside board_ctl_set_vco().
}

void prog_dds1_ftw(const ftw_t * value, unsigned char freqpow, unsigned char ddsmult)
{
	ftw_t val;
	//phase_copy(& val, value);
	if (freqpow != 0)
	{
		val = * value << freqpow;
		value = & val;
	}

#if (DDS1_TYPE == DDS_TYPE_AD9852)
	prog_pulse_ioreset();
	prog_ad9852_freq1(targetdds1, value);
	prog_pulse_ioupdate();
#elif (DDS1_TYPE == DDS_TYPE_AD9857)
	prog_pulse_ioreset();
	prog_ad9857_freq(targetdds1, 0x00, value);
	prog_pulse_ioupdate();
#elif (DDS1_TYPE == DDS_TYPE_AD9951)
	prog_pulse_ioreset();
	prog_ad9951_freq0(targetdds1, value);
	prog_pulse_ioupdate();
#elif (DDS1_TYPE == DDS_TYPE_AD9834)
	prog_ad9834_freq(targetdds1, value);
#elif (DDS1_TYPE == DDS_TYPE_AD9851)
	prog_ad9851_freq(targetdds1, value);
	prog_pulse_ioupdate();
#elif (DDS1_TYPE == DDS_TYPE_AD9832)
	prog_ad9832_freq(targetdds1, value);
	prog_pulse_ioupdate();
#endif
}

void prog_dds2_ftw(const ftw_t * value, unsigned char freqpow, unsigned char ddsmult)
{
	ftw_t v;
	if (freqpow != 0)
	{
		v = * value << freqpow;
		value = & v;
	}

#if (DDS2_TYPE == DDS_TYPE_AD9852)
	prog_pulse_ioreset();
	prog_ad9852_freq1(targetdds2, value);
	prog_pulse_ioupdate();
#elif (DDS2_TYPE == DDS_TYPE_AD9857)
	prog_pulse_ioreset();
	prog_ad9857_freq(targetdds2, 0x00, value);
	prog_pulse_ioupdate();
#elif (DDS2_TYPE == DDS_TYPE_AD9951)
	prog_pulse_ioreset();
	prog_ad9951_freq0(targetdds2, value);
	prog_pulse_ioupdate();
#elif (DDS2_TYPE == DDS_TYPE_AD9834)
	prog_ad9834_freq(targetdds2, value);
#elif (DDS2_TYPE == DDS_TYPE_AD9851)
	prog_ad9851_freq(targetdds2, value);
	prog_pulse_ioupdate();
#elif (DDS2_TYPE == DDS_TYPE_AD9832)
	prog_ad9832_freq(targetdds2, value);
	prog_pulse_ioupdate();
#endif
}

static const phase_t phase_100 = 100;

static void prog_pll1_init(void)
{
#if defined(PLL1_TYPE) && (PLL1_TYPE == PLL_TYPE_LMX2306)
	prog_lmx2306_init(targetpll1);
	prog_lmx2306_func(targetpll1, 0, 0x00);	/* set phase det polarity */
	prog_lmx2306_r(targetpll1, & r1_ph);	/* Set divider F osc R - 3...16383 */
	prog_lmx2306_n(targetpll1, & phase_100, PLL1_GO_STATE);	/* Set divider F osc R - 3...16383 */
#elif defined(PLL1_TYPE) && (PLL1_TYPE == PLL_TYPE_ADF4001)
	prog_adf4001_init(targetpll1);
	prog_adf4001_func(targetpll1, 0, 0x00, PLL1_CPI);	/* set phase det polarity */
	prog_adf4001_r(targetpll1, & r1_ph);	/* Set divider F osc R - 3...16383 */
	prog_adf4001_n(targetpll1, & phase_100, PLL1_GO_STATE);	/* Set divider F osc R - 3...16383 */
#elif defined(PLL1_TYPE) && (PLL1_TYPE == PLL_TYPE_MC145170)
	prog_mc145170_init(targetpll1);
	prog_mc145170_func(targetpll1, !0);	/* set phase det polarity */
	prog_mc145170_r(targetpll1, & r1_ph);	/* Set divider F osc R - 3...16383 */
	prog_mc145170_n(targetpll1, & phase_100);	/* Set divider F osc R - 3...16383 */
#endif
	phase_last_n1 = 100;
}

void prog_pll1_n(const phase_t * value)
{
	if (phase_last_n1 == * value)
		return;
	phase_last_n1 = * value;

#if (PLL1_TYPE == PLL_TYPE_LMX2306)
	prog_lmx2306_n(targetpll1, value, PLL1_GO_STATE);		/* set divider f in  n - 56..65535 */
#elif (PLL1_TYPE == PLL_TYPE_ADF4001)
	prog_adf4001_n(targetpll1, value, PLL1_GO_STATE);		/* set divider f in  n - 56..65535 */
#elif (PLL1_TYPE == PLL_TYPE_MC145170)
	prog_mc145170_n(targetpll1, value);		/* set divider f in  n - 56..65535 */
#endif
}


static void prog_pll2_init(void)
{
	unsigned char target = targetpll2;
#if defined(PLL2_TYPE) && (PLL2_TYPE == PLL_TYPE_LMX2306)
	prog_lmx2306_init(target);		
	prog_lmx2306_func(target, 1, 0x00);	/* set phase det polarity */
	prog_lmx2306_r(target, & phase_lo2_r);	/* Set divider F osc R - 3...16383 */
	prog_lmx2306_n(target, & phase_lo2_n, PLL3_GO_STATE);	/* Set divider F in  N - 56..65535 */
#elif defined(PLL2_TYPE) && (PLL2_TYPE == PLL_TYPE_ADF4001)
	prog_adf4001_init(target);		
	prog_adf4001_func(target, 1, 0x00, PLL2_CPI);	/* set phase det polarity */
	prog_adf4001_r(target, & phase_lo2_r);	/* Set divider F osc R - 3...16383 */
	prog_adf4001_n(target, & phase_lo2_n, PLL3_GO_STATE);	/* Set divider F in  N - 56..65535 */
#elif defined(PLL2_TYPE) && (PLL2_TYPE == PLL_TYPE_MC145170)
	prog_mc145170_init(target);		
	prog_mc145170_func(target, !1);	/* set phase det polarity */
	prog_mc145170_r(target, & phase_lo2_r);	/* Set divider F osc R - 3...16383 */
	prog_mc145170_n(target, & phase_lo2_n);	/* Set divider F in  N - 56..65535 */
#endif
}

static void prog_dds1_init(unsigned char ddsmult)
{
#if defined(DDS1_TYPE) && (DDS1_TYPE == DDS_TYPE_AD9852)
	prog_ad9852_init(targetdds1, 0, ddsmult);
#elif defined(DDS1_TYPE) && (DDS1_TYPE == DDS_TYPE_AD9857)
	prog_ad9857_init(targetdds1, 0, ddsmult);
#elif defined(DDS1_TYPE) && (DDS1_TYPE == DDS_TYPE_AD9951)
	prog_ad9951_init(targetdds1, 0, ddsmult, DDSAD9951CLKGT250);
#elif defined(DDS1_TYPE) && (DDS1_TYPE == DDS_TYPE_AD9851)
	prog_ad9851_init(targetdds1, 0, ddsmult);
#elif defined(DDS1_TYPE) && (DDS1_TYPE == DDS_TYPE_AD9834)
	prog_ad9834_init(targetdds1);
#elif defined(DDS1_TYPE) && (DDS1_TYPE == DDS_TYPE_AD9832)
	prog_ad9832_init(targetdds1);
#endif
}

static void prog_dds2_init(unsigned char ddsmult)
{
#if defined(DDS2_TYPE) && (DDS2_TYPE == DDS_TYPE_AD9852)
	prog_ad9852_init(targetdds2, 0, ddsmult);
#elif defined(DDS2_TYPE) && (DDS2_TYPE == DDS_TYPE_AD9857)
	prog_ad9857_init(targetdds2, 0, ddsmult);
#elif defined(DDS2_TYPE) && (DDS2_TYPE == DDS_TYPE_AD9951)
	prog_ad9951_init(targetdds2, 0, ddsmult, DDSAD9951CLKGT250);
#elif defined(DDS2_TYPE) && (DDS1_TYPE == DDS_TYPE_AD9851)
	prog_ad9851_init(targetdds2, 0, ddsmult);
#elif defined(DDS2_TYPE) && (DDS2_TYPE == DDS_TYPE_AD9834)
	prog_ad9834_init(targetdds2);
#elif defined(DDS2_TYPE) && (DDS2_TYPE == DDS_TYPE_AD9832)
	prog_ad9832_init(targetdds2);
#endif
}

static void prog_amps_init(void)
{
#if defined(AMP1_TYPE) && (AMP1_TYPE == AMP_TYPE_AD8320)
	/* LO1 amplifier */
	prog_ad8320(targetamp1, (unsigned char) iAmp1Level);	//
	board_set_loamp1(0x01);
	board_set_allamp(0x01);
#endif

#if defined(AMP2_TYPE) && (AMP2_TYPE == AMP_TYPE_AD8320)
	/* LO2 amplifier */
	prog_ad8320(targetamp2, (unsigned char) iAmp2Level);	//
	board_set_loamp2(0x01);
	board_set_allamp(0x01);
#endif

#if defined(AMP3_TYPE) && (AMP3_TYPE == AMP_TYPE_AD8320)
	/* LO3 amplifier */
	prog_ad8320(targetamp3, (unsigned char) iAmp3Level);	//
	board_set_loamp3(0x01);
	board_set_allamp(0x01);
#endif
}

void board_init_io(void)
{
	prog_select_init();		// spi initialize
	prog_pulse_ioreset();
	board_ctl_init();
}
/* Initialize chips. All coeffecienters should be already calculated before. */
void board_init(unsigned char ddsmult)
{

	phase_last_n1 = 0;

	phase_lo2_r = LO2_PLL_R;
	phase_lo2_n = LO2_PLL_N;

	prog_dds1_init(ddsmult);
	prog_dds2_init(ddsmult);

	prog_amps_init();

	// calculate global constants - 
	// setup scaler PLL - 1-st lo
	prog_pll1_init();
	// setup fixed PLL - 2-nd lo
	prog_pll2_init();
	/* setup VFOs limits */

	unsigned long tmp;
	tmp = SYNTH_FREQMIN;	/* scale freq before calculate phase */
	fastcount_t i;
	for (i = 0; i < sizeof board_vfos / sizeof board_vfos [0]; ++ i)
	{

		tmp = (unsigned long) ((phase_t)tmp * FREQ_SCALE / FREQ_DENOM);
		board_vfos [i] = (int16_t) (tmp >> VFODIVPOWER);
	}
}



#endif /* BOARD_SUPPORT_UA1ARN */

