#ifndef AD5932_H_INCLUDED
#define AD5932_H_INCLUDED

/* programming AD5932 */

static void prog_ad5932_pair(
	unsigned char target,	/* addressing to chip */
	unsigned char addess,	/* chip register - LSB */
	const phase_t * val,
	fastcount_t	left		/* LEFT one position */
	)
{
	// Set LSBs
	prog_select(target, 1);	/* start sending data to target chip */

	prog_val(target, addess + 0x00, 4);		/* D15, D14, D13, D12 */
	prog_phbits(target, val, left - 12, 12);

	// Set MSBs
	prog_val(target, addess + 0x01, 4);		/* D15, D14, D13, D12 */
	prog_phbits(target, val, left - 0, 12);

	prog_select(target, 0);	/* done sending data to target chip */
}

/* set frequency tuning word1 to DDS */
static void prog_ad5932_freq(
	unsigned char target,	/* addressing to chip */
	const phase_t * val
	)
{

	assert(PHASE_RESOLUTION >= 24);
	prog_ad5932_pair(target, 0x0c, val, PHASE_RESOLUTION);
}


/* set frequency tuning word1 to DDS */
static void prog_ad5932_incr(
	unsigned char target	/* addressing to chip */
	)
{
	prog_ad5932_pair(target, 0x02, & phase_0, 24);
}

/* set number of uncrement intervals */
static void prog_ad5932_numincr(
	unsigned char target	/* addressing to chip */
	)
{
	// Set LSBs
	prog_select(target, 1);	/* start sending data to target chip */

	prog_val(target, 0x01, 4);		/* D15, D14, D13, D12 */
	prog_phbits(target, & phase_2, 12, 12);

	prog_select(target, 0);	/* done sending data to target chip */
}

/* set increment interval time */
static void prog_ad5932_incitervaltime(
	unsigned char target	/* addressing to chip */
	)
{
	// Set increment interval same as MCLK
	prog_select(target, 1);	/* start sending data to target chip */

	prog_val(target, 0x03, 3);		/* D15, D14, D13 */
	prog_val(target, 0x00, 2);		/* D12, D11 - Multiply (1/MCLK) by 1 */
	prog_phbits(target, & phase_2, 11, 11);
	
	prog_select(target, 0);	/* done sending data to target chip */
}

/* complex init */


void prog_ad5932_init(
	unsigned char target,	/* addressing to chip */
	unsigned char powerdown
			 )
{
	prog_ad5932_incr(target);
	prog_ad5932_numincr(target);
	prog_ad5932_incitervaltime(target);

	// Set control word
	prog_select(target, 1);	/* start sending data to target chip */

	prog_val(target, 0x00, 4);		/* D15, D14, D13, D12 */

	prog_bit(target, 1);	/* D11: B24 = 1	*/
	prog_bit(target, 1);	/* D10: DAC ENABLE = 1	*/
	prog_bit(target, 1);	/* D9: SINE		*/
	prog_bit(target, 0);	/* D8: MSBOUTEN		*/

	prog_bit(target, 1);	/* D7: reserved		*/
	prog_bit(target, 1);	/* D6: reserved		*/
	prog_bit(target, 0);	/* D5: external int trigger freq increment		*/
	prog_bit(target, 1);	/* D4: reserved		*/

	prog_bit(target, 0);	/* D3: SYNCSEL		*/
	prog_bit(target, 0);	/* D2: SYNCOUTEN		*/
	prog_bit(target, 1);	/* D1: reserved		*/
	prog_bit(target, 1);	/* D0: reserved		*/

	prog_select(target, 0);	/* done sending data to target chip */

}

#endif /* AD5932_H_INCLUDED */
