#ifndef AD9951_H_INCLUDED
#define AD9951_H_INCLUDED

extern const phase_t phase_0;

#define OUTSHAPE9951 0	/* Use OSK multiplier with 0x0fc0 */
#define OUTSHAPE9951VAL 0x0400 //0x0fc0

/* programming AD9951/AD9854 */

static void prog_ad9951(
	unsigned char target,	/* addressing to chip */
	unsigned char address,	/* serial adress value */
	const phase_t * val,
	fastcount_t n				/* number of bits to send */
	)
{
	prog_select(target, 1);	/* start sending data to target chip */

	prog_val(target, 0, 3);		/* write, x, x, x */
	prog_val(target, address, 5);		/* A4, A3 ,A2, A1, A0 */
	prog_phbits(target, val, n, n);

	prog_select(target, 0);	/* done sending data to target chip */

}

static void prog_ad9951_fill(
	unsigned char target,	/* addressing to chip */
	unsigned char address,	/* serial adress value */
	const phase_t * val,
	fastcount_t n,			/* number of leading bits to send */
	fastcount_t z			/* number of trailing zeroes to send */
	)
{
	prog_select(target, 1);	/* start sending data to target chip */

	prog_val(target, 0, 3);		/* write, x, x, x */
	prog_val(target, address, 5);		/* A3 ,A2, A1, A0 */
	prog_phbits(target, val, n, n);
	prog_phbits(target, & phase_0, z, z);

	prog_select(target, 0);	/* done sending data to target chip */

}

/* set frequency tuning word1 to DDS */
static void prog_ad9951_freq0(
	unsigned char target,	/* addressing to chip */
	const phase_t * val
	)
{
	enum { r = PHASE_RESOLUTION < 32 ? PHASE_RESOLUTION : 32 };
	enum { z = 32 - r };
	/* Target register - FTW0 (0x04) */
	prog_ad9951_fill(target, 0x04, val, r, z);

}


/* set control word CFR1 */
static void prog_ad9951_cfr1(
	unsigned char target,	/* addressing to chip */
	const phase_t * val
	)
{
	prog_ad9951(target, 0x00, val, 32);
}

/* set control word CFR2 */
static void prog_ad9951_cfr2(
	unsigned char target,	/* addressing to chip */
	const phase_t * val
	)
{
	prog_ad9951(target, 0x01, val, 24);
}

#if 0

/* set control word ASF - Amplitude Scale Factor */
/* 12 bit of value has meaning	*/
static void prog_ad9951_asf(
	unsigned char target,	/* addressing to chip */
	const phase_t * val
	)
{
	prog_ad9951(target, 0x02, val, 16);
}
#endif
/* complex init */


void prog_ad9951_init(
	unsigned char target,	/* addressing to chip */
	unsigned char powerdown,
	unsigned fqscale,
	unsigned vcogt250		/* cfr2<2> if freqscale != 0 and internal freq > 250 MHz */
			 )
{
	//return;
	//   ad9854
	// 52  54   
	if (powerdown != 0)
	{
		/* sleep mode - offline everything */
		phase_t ctl1, ctl2;
		phase_fromulong(& ctl1, 
			(0x17UL << 24) |		// init - 0x10
			(0x20UL << 16) |		// init - 0x64
			(0x00UL << 8) |		// init - 0x01	10 - qdac 00 - control dac
			0x40UL //(0x60 << 0)			// 0x40 - bypass inv sinc, 0x20 - osk enable
			);

		prog_pulse_ioreset();
		prog_ad9951_cfr2(target, & ctl2);
		prog_ad9951_cfr1(target, & ctl1);
		prog_pulse_ioupdate();
#if 1
		//          
		// io update clock,    .
		//  -  
		prog_ad9951_cfr2(target, & ctl2);
		prog_ad9951_cfr1(target, & ctl1);
		prog_pulse_ioupdate();
#endif
	}
	else
	{
		phase_t ctl1, ctl2;
		if (fqscale == 1)
		{
			/* no internal multiplier, PLL bypassed */
			phase_fromulong(& ctl1, 
				(0x00L << 24) |
				(0x00L << 16) |
				(0x02L << 8) |
				(0x02L << 0)
				);
			phase_fromulong(& ctl2, 
				(0x00L << 16) |
				(0x00L << 8) |
				(0x00L << 0)
				);
		}
		else
		{
			unsigned icp = 0x03;	/* cfr2<1:0> */
			/* With internal multiplier */
			phase_fromulong(& ctl1, 
				(0x00L << 24) |
				(0x00L << 16) |
				(0x02L << 8) |
				(0x02L << 0)
				);
			phase_fromulong(& ctl2, 
				(0x00L << 16) |
				(0x00L << 8) |
				(((fqscale << 3) | (vcogt250 ? 0x04L : 0x00L) | icp) << 0)
				);
		}
		prog_pulse_ioreset();
		prog_ad9951_cfr2(target, & ctl2);
		prog_ad9951_cfr1(target, & ctl1);
		prog_pulse_ioupdate();
#if 1
		//          
		// io update clock,    .
		//  -  
		prog_ad9951_cfr2(target, & ctl2);
		prog_ad9951_cfr1(target, & ctl1);
		prog_pulse_ioupdate();
#endif
	}

	//  update clock rate
#if TESTPOINTS
	{
		phase_t upd;

		phase_fromuint(& upd, 0x40);
		prog_ad9951_update_clock(target, & upd);
		prog_pulse_ioupdate();
	}
#endif	
	//   
#if OUTSHAPE9951
	{
		phase_t dmult;
		//
		//     0x0fff
		//      
		//
		phase_fromuint(& dmult, OUTSHAPE9951VAL);
		prog_ad9951_asf(target, & dmult);
		// ad9854 specific
		//prog_ad9951_output_shape_key_Q(target, & dmult);
		prog_pulse_ioupdate();
	}
#endif

}

#endif /* AD9951_H_INCLUDED */
