#ifndef LMX2306_H_INCLUDED
#define LMX2306_H_INCLUDED

/* programming LMX2306 */
static void prog_lmx2306_init(
	unsigned char target	/* addressing to chip */
	)
{

	prog_select(target, 1);

	prog_bit(target, 1);	/* c2 bit */
	prog_bit(target, 1);	/* c1 bit */

	prog_select(target, 0);
}

static void prog_lmx2306_func(
	unsigned char target,	/* addressing to chip */
	unsigned char polarity,
	unsigned char FoLD		/* F5..F3: Fo/LD: 4 - output of R divider, 2 - output of N divider  */
	)
{
	prog_select(target, 1);	/* start sending data to target chip */

	prog_bit(target, 0);	/* F19: test mode should be 0 */
	prog_bit(target, 0);	/* F18: power down mode = 0 */
	prog_val(target, 0, 3);	/* F17..F15: test mode should be 0 */
	prog_val(target, 0, 4);	/* F14..F11: timeout counter */
	prog_val(target, 0x00, 3);	/* F10..F8: fastlock modes */
	prog_bit(target, 0);	/* F7: CP tri-state */
	prog_bit(target, polarity);	/* F6: phase detector polarity */
	prog_val(target, FoLD, 3);	/* F5..F3: Fo/LD: 4 - output of R divider, 2 - output of N divider  */
	prog_bit(target, 0);	/* F2: power down */
	prog_bit(target, 0x00);	/* F1: counter reset */

	prog_bit(target, 1);	/* C2: bit */
	prog_bit(target, 0);	/* C1: bit */

	prog_select(target, 0);
}

/* Set divider F osc R - 3...16383 */
void prog_lmx2306_r(
	unsigned char target,	/* addressing to chip */
	const phase_t * r		/* divider F osc R - 3...16383 */
	)
{
	prog_select(target, 1);	/* start sending data to target chip */

	prog_bit(target, 0);	/* LD precision */

	prog_val(target, 0, 4);	/* R18..F15: test mode should be 0 */
	prog_phbits(target, r, 14, 14);

	prog_bit(target, 0);	/* c2 bit */
	prog_bit(target, 0);	/* c1 bit */

	prog_select(target, 0);
}

/*
	N19 determines the charge pump current magnitude 
	
*/

/* Set divider F in  N - 56..65535 */
static void prog_lmx2306_n(
	unsigned char target,	/* addressing to chip */
	const phase_t * n,		/* divider F in  N - 56..65535 */
	unsigned char go
	)
{
	prog_select(target, 1);	/* start sending data to target chip */
							/* N19=LOW ICPo = 250 A, N19=HIGH ICPo = 1 mA. */
	prog_bit(target, go);	/* GO bit - for fast changing frequency w/o lost synch */
	prog_phbits(target, n, 16, 13);	/* B bits */
	prog_phbits(target, n, 5, 5);		/* A bits */
	prog_bit(target, 0);	/* c2 bit */
	prog_bit(target, 1);	/* c1 bit */

	prog_select(target, 0);
}

#endif /* LMX2306_H_INCLUDED */