#ifndef MC145170_H_INCLUDED
#define MC145170_H_INCLUDED

/*
	Programming MC145170D2
	*/

static void prog_mc145170_init(
	unsigned char target	/* addressing to chip */
	)
{
	prog_select(target, 1);

	prog_val(target, 0x02, 5);	/* Initialization sequence */

	prog_select(target, 0);
}


static void prog_mc145170_func(
	unsigned char target,	/* addressing to chip */
	unsigned char pol		/* ph det polarity: 0- inv, 1 - direct */
	)
{
	prog_select(target, 1);

	prog_bit(target, pol);		/* C7: POL (1-with invertor, 0 - direct)	*/
	prog_bit(target, 0x01);		/* C6: PDA/B - 0x00 - A, 0x01 - B (tri-state)	*/
	prog_bit(target, 0x00);		/* C5: LDE		*/
#if TESTPOINTS
	prog_val(target, 0x01, 3);	/* C4..C2: OSC0 0 - low, 2 - osc/2	*/
	prog_bit(target, 0x01);		/* C1: Enables Fv output */
	prog_bit(target, 0x01);		/* C0: Enables Fr output */
#else
	prog_val(target, 0x00, 3);	/* C4..C2: OSC0 0 - low, 2 - osc/2	*/
	prog_bit(target, 0x00);		/* C1: Enables Fv output */
	prog_bit(target, 0x00);		/* C0: Enables Fr output */
#endif

	prog_select(target, 0);
}

/* Set divider F osc R - 1, 5..32767 */
void prog_mc145170_r(
	unsigned char target,	/* addressing to chip */
	const phase_t * r		/* divider F osc R - 1, 5..32767 */
	)
{
	prog_select(target, 1);	/* start sending data to target chip */

	prog_phbits(target, r, 15, 15);	/* R bits */

	prog_select(target, 0);
}

/* Set divider F in  N - 40..65535 */
static void prog_mc145170_n(
	unsigned char target,	/* addressing to chip */
	const phase_t * n		/* divider F in  N - 56..65535 */
	)
{
	prog_select(target, 1);	/* start sending data to target chip */

	prog_phbits(target, n, 16, 16);	/* B bits */

	prog_select(target, 0);
}

#endif /* MC145170_H_INCLUDED */
