#ifndef AD9834_H_INCLUDED
#define AD9834_H_INCLUDED

/* programming AD9834 */

/* complex init */

/* internal function */
/* this function is'not control chip select/fsync line */
static void prog_ad9834_control(
	unsigned char target,	/* addressing to chip */
	fastbool_t reset,	/* 0/1 */
	fastbool_t hlb,	/* 0/1 */
	fastbool_t fsel		/* freq profile for output 0/1 */
	)
{
	// Set control word

	prog_val(target, 0x00, 2);		/* D15, D14: 0 = control register address */

	prog_bit(target, 1);	/* D13: B28 = 1	*/
	prog_bit(target, hlb);	/* D12: HLB	*/

	prog_bit(target, fsel);	/* D11: FSEL = 0	*/
	prog_bit(target, 0);	/* D10: PSEL = 0	*/
	prog_bit(target, 0);	/* D9: PIN=1, SW=0		*/
	prog_bit(target, reset);	/* D8: RESET		*/

	prog_bit(target, 0);	/* D7: SLEEP1 (STOP MCLK)		*/
	prog_bit(target, 0);	/* D6: SLEEP2 (PD DAC)		*/
	prog_bit(target, 0);	/* D5: OPBITEN	*/
	prog_bit(target, 0);	/* D4: SIGN/PB		*/

	prog_bit(target, 0);	/* D3: DIV2 (divide NCO output)		*/
	prog_bit(target, 0);	/* D2: reserved		*/
	prog_bit(target, 0);	/* D1: MODE (0=sin, 1=triangle)		*/
	prog_bit(target, 1);	/* D0: reserved		*/

}

/* internal data */
/* last profile code, used for generate frequency */
static fastbool_t prog_ad9834_active_profile [8];

/* set frequency tuning word1 to DDS */
static void prog_ad9834_freq(
	unsigned char target,	/* addressing to chip */
	const phase_t * val
	)
{
	assert(target < 8);
	fastbool_t current = prog_ad9834_active_profile [target] != 0;
	fastbool_t fselnext = ! current;

	//current = 0;
	//fselnext = 0;
	/* 0x01: freq0, 0x02: freq1 */ 
	unsigned char freqx = fselnext ? 0x02 : 0x01;

	// assume - last initialization match to active_profile data.

	// Set LSBs
	prog_select(target, 1);	/* start sending data to target chip */

	prog_val(target, freqx, 2);		/* D15, D14 */
	assert(PHASE_RESOLUTION >= 28);
	prog_phbits(target, val, PHASE_RESOLUTION - 14, 14);

	prog_val(target, freqx, 2);		/* D15, D14 */
	assert(PHASE_RESOLUTION >= 28);
	prog_phbits(target, val, PHASE_RESOLUTION - 0, 14);


	/* switch output for text profile */
	prog_ad9834_control(target, 0, 0, fselnext);

	prog_select(target, 0);	/* done sending data to target chip */

	prog_ad9834_active_profile [target] = fselnext;

}

void prog_ad9834_init(
	unsigned char target	/* addressing to chip */
			 )
{
	enum { fsel = 0 };
	prog_select(target, 1);	/* start sending data to target chip */

	prog_ad9834_control(target, 1, 0, fsel);
	prog_ad9834_control(target, 0, 0, fsel);

	prog_select(target, 0);	/* done sending data to target chip */

	prog_ad9834_active_profile [target] = fsel;

}
/*
void prog_ad9834_clearlow(
	phase_t * val
	)
{
#if PHASE_RESOLUTION > 28
	phase_t mask;
	unsigned long lmask = ~ ((1UL << (PHASE_RESOLUTION - 28)) - 1);
	phase_fromulong(& mask, lmask);
	phase_and2(val, & mask);
#endif
}
*/


#endif /* AD9834_H_INCLUDED */
