/* bandfilters.c */

#include <stdlib.h>
#include <assert.h>

#include "board.h"
#include "product.h"
#include "synthcalcs.h"

/* Pin manipulation functions */
#include <avr/io.h>
#include <util/delay.h>
//#include <avr/pgmspace.h>


#if defined (__AVR_ATmega128__)

#define TARGET_BANDF_PORT PORTE
#define TARGET_BANDF_DDR PORTE
#define TARGET_BANDF_PIN PORTE

#define BANDMASK 0x07
#define BANDSHIFT 4

#elif defined (__AVR_ATmega32__) || defined (__AVR_ATmega16__)

#define TARGET_BANDF_PORT PORTD
#define TARGET_BANDF_DDR DDRD
#define TARGET_BANDF_PIN PIND

#define BANDMASK 0x07
#define BANDSHIFT 4

#endif

/* set band filters */

/* for version 0...1.5, 1.5...60.0 MHz - 12 bands. */
#define BANDF_FREQMIN 1500000UL
#define BANDF_FREQ_SCALE 1398433UL
#define BANDF_FREQ_DENOM 1000000UL

#define BANDMASK 0x07
#define BANDSHIFT 4

#define BANDDIVPOWER	12	/* ~4 kHz granulation */

static uint16_t board_bandfs [10];
static uint8_t bandsels [12];	// zero position and last position is out of values in table

void board_bandf_initialize()
{
	uint32_t tmp;
	tmp = BANDF_FREQMIN;	/* scale freq before calculate phase */
	fastcount_t i;
	for (i = 0; i < sizeof board_bandfs / sizeof board_bandfs [0]; ++ i)
	{

		tmp = (unsigned long) ((phase_t)tmp * BANDF_FREQ_SCALE / BANDF_FREQ_DENOM);
		board_bandfs [i] = (uint16_t) (tmp >> BANDDIVPOWER);
	}

#if CANADAPSEL

	bandsels [0] = 0x00;	// on output if less then 1.5 MHz

	board_bandfs [0] = (uint16_t) (2200000 >> BANDDIVPOWER);
	bandsels [1] = 0x00;	// on output if less then 2.2 MHz

	board_bandfs [1] = (uint16_t) (5500000 >> BANDDIVPOWER);
	bandsels [2] = 0x01;	// on output if less then 5.5 MHz

	board_bandfs [2] = (uint16_t) (8500000 >> BANDDIVPOWER);
	bandsels [3] = 0x02;	// on output if less then 8.5 MHz

	board_bandfs [3] = (uint16_t) (12000000 >> BANDDIVPOWER);
	bandsels [4] = 0x05;	// on output if less then 12.0 MHz

	board_bandfs [4] = (uint16_t) (17000000 >> BANDDIVPOWER);
	bandsels [5] = 0x06;	// on output if less then 17.0 MHz

	board_bandfs [5] = (uint16_t) (24000000 >> BANDDIVPOWER);
	bandsels [6] = 0x07;	// on output if less then 24.0 MHz

	board_bandfs [6] = (uint16_t) (33000000 >> BANDDIVPOWER);
	bandsels [7] = 0x03;	// on output if less then 33.0 MHz

	board_bandfs [7] = (uint16_t) (33000000 >> BANDDIVPOWER);
	bandsels [8] = 0x03;	// on output if less then 33.0 MHz

	board_bandfs [8] = (uint16_t) (33000000 >> BANDDIVPOWER);
	bandsels [9] = 0x03;	// on output if less then 33.0 MHz

	board_bandfs [9] = (uint16_t) (33000000 >> BANDDIVPOWER);
	bandsels [10] = 0x03;	// on output if less then 33.0 MHz 

	bandsels [11] = 0x04;	// on output if greater then 33.0 MHz 

#endif // CANADAPSEL


/*       
*/
#if defined (__AVR_ATmega128__)
	TARGET_BANDF_DDR |= (BANDMASK << BANDSHIFT);	// define these bits as outputs
#elif defined (__AVR_ATmega32__) || defined (__AVR_ATmega16__)
	TARGET_BANDF_DDR |= (BANDMASK << BANDSHIFT);	// define these bits as outputs
#endif

}


void board_bandf_setfreq(
	unsigned long freq
	)
{
  uint16_t freqloc = (uint16_t) (freq >> BANDDIVPOWER);
	unsigned char i;
	if (freqloc < (uint16_t) (BANDF_FREQMIN >> BANDDIVPOWER))
		i = 0x00;
	else
	{



		for (i = 0; i < sizeof board_bandfs / sizeof board_bandfs [0]; ++ i)
		{
			if (board_bandfs [i] >= freqloc)
				break;
		}
		i += 1;
	}
	uint8_t bandsel = bandsels [i] & BANDMASK;
	// todo: here i contain required band filter select code.
#if defined (__AVR_ATmega128__)
	TARGET_BANDF_PORT = (PORTD & ~ (BANDMASK << BANDSHIFT)) | bandsel << BANDSHIFT;
#elif defined (__AVR_ATmega32__) || defined (__AVR_ATmega16__)
	TARGET_BANDF_PORT = (PORTD & ~ (BANDMASK << BANDSHIFT)) | bandsel << BANDSHIFT;
#endif
}
