//#include <stdio.h>
//#include <stdlib.h>
//#include <math.h>


#include <stdint.h>
#include <stdlib.h>

#include "encoder.h"

#include <avr/interrupt.h>
#include <avr/io.h>
//#include <avr/pgmspace.h>
#include <util/delay.h>


#define ENCODER_MULTICLICK	1	/*     RU3GA -     */

#define TICKCOUNT_MAX 255
static volatile uint8_t tickcount;

// Timer 0 overflow interrupt service routine
// 7.813 kHz / 256 = approx 30 Hz
void	enc_spool(void)
{
	if (tickcount != TICKCOUNT_MAX)
		++ tickcount;
}


void	kbd_spool(void);

#if ENCODER_MULTICLICK

#define ROTATE_LORES_DIV (4 * 4)
#define ROTATE_LORES_DIV2 (2 * 4)

#else

#define ROTATE_LORES_DIV 4
#define ROTATE_LORES_DIV2 2

#endif

/*     */

/*     -   */
static uint16_t stepcount;


static volatile int16_t rotate;


#if ENCODER_MULTICLICK

//static uint8_t old_val1;
//static uint8_t old_val2;
static uint8_t old_val;

static void spool_encinterrupt(void)
  {        

	
#if defined (__AVR_ATmega128__)
	uint8_t new_val = (PINE & ((1 << PE5) | (1 << PE4))) >> PE4;

#elif defined (__AVR_ATmega32__) || defined (__AVR_ATmega16__)
	//const uint8_t bits = PIND; // & ((1 << PD3) | (1 << PD2));
	//uint8_t new_val1 = bits & (1 << PD3);
	//uint8_t new_val2 = bits & (1 << PD2);
	uint8_t new_val = (PIND & ((1 << PD3) | (1 << PD2))) >> PD2;
#endif

	// dimensions are:
	// old_bits new_bits
	const static signed char v [4][4] =
	{
		{
			+0,		/* 00 -> 00 stopped */
			-1,		/* 00 -> 01 rotate left */
			+1,		/* 00 -> 10 rotate right */
			+0,		/* 00 -> 11 invalid combination */		
		},
		{
			+1,		/* 01 -> 00 rotate right */
			+0,		/* 01 -> 01 stopped */
			+0,		/* 01 -> 10 invalid combination */
			-1,		/* 01 -> 11 rotate left */
		},
		{
			-1,		/* 10 -> 00 rotate left */
			+0,		/* 10 -> 01 invalid combination */
			+0,		/* 10 -> 10 stopped */
			+1,		/* 10 -> 11 rotate right */
		},
		{
			+0,		/* 11 -> 00 invalid combination */
			+1,		/* 11 -> 01 rotate right */
			-1,		/* 11 -> 10 rotate left */
			+0,		/* 11 -> 11 stopped */
		},
	};


	rotate += v [old_val][new_val];

	old_val = new_val;
}


#if defined (__AVR_ATmega128__)

ISR(INT4_vect)
{
	spool_encinterrupt();
}

ISR(INT5_vect)
{
	spool_encinterrupt();
}

#else

ISR(INT0_vect)
{
	spool_encinterrupt();
}

ISR(INT1_vect)
{
	spool_encinterrupt();
}

#endif

#else

// int0 input = pd2
// int1 input = pd3
ISR(INT0_vect)
{
	rotate -= 1;
}

ISR(INT1_vect)
{
	rotate += 1;
}

#endif

void encoder_clear(void)
{
	cli();
	rotate = 0;
	stepcount = 0;
	tickcount = TICKCOUNT_MAX;
	sei();
}


/*      . 
		  */
int16_t 
getRotateHiRes(
	uint32_t * jumpsize, 
	uint16_t granulation)
{
#if ENCODER_MULTICLICK
#define BIGJUMPSIZE (10000UL / 4)
	static const uint8_t velotable [] =
	{
		1, 1, 1, 1, 1, 1, 
		1, 1, 1, 1, 1, 1, 
		1, 1, 1, 1, 1, 1, 
		1, 1, 1, 1, 1, 1, 
		2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2,
		2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2,
		2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2,
		2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2,
	};
#else
#define BIGJUMPSIZE (10000UL)
	static const uint8_t velotable [] =
	{
		1, 1, 1, 1, 1, 1, 
		2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2,
	};
#endif

	div_t d;
	uint8_t ticks;
	uint16_t nrotate;
	cli();
	if (tickcount != 0)
	{
		ticks = tickcount;
		tickcount = 0;
		nrotate = rotate;
		rotate = 0;
	}
	else
	{
		ticks = 1;
		nrotate = 0;
	}
	sei();

	d = div(nrotate + stepcount, ticks);
	stepcount += d.rem;	/*      */

	if (d.quot < 0)
		d.quot = - d.quot;
	if (d.quot < (sizeof velotable / sizeof velotable [0]))
		* jumpsize = (uint32_t) granulation * velotable [d.quot];
	else 
		* jumpsize = 10000UL;
		

	return nrotate;
}

/*  ""    .
 *       scale,    
 */

int getRotateLoRes(void)
{
	int nrotate;
	div_t d;
	cli();
	nrotate = rotate;
	rotate = 0;
	sei();

	d = div(nrotate, ROTATE_LORES_DIV);

	cli();
	rotate += d.rem;
	sei();

	return d.quot;
}


/*  ""    .
 *       scale,    
 */

int getRotateLoRes2(void)
{
	int nrotate;
	div_t d;
	cli();
	nrotate = rotate;
	rotate = 0;
	sei();

	d = div(nrotate, ROTATE_LORES_DIV2);

	cli();
	rotate += d.rem;
	sei();

	return d.quot;
}

void encoder_initialize(void)
{
	rotate = 0;
	stepcount = 0;
	tickcount = TICKCOUNT_MAX;

#if defined (__AVR_ATmega128__)

#if ENCODER_MULTICLICK

// External Interrupt(s) initialization
// INT0: Off
// INT1: Off
// INT2: Off
// INT3: Off
// INT4: On
// INT4 Mode: Any change
// INT5: On
// INT5 Mode: Any change
// INT6: Off
// INT7: Off
EICRA = 0x00;
EICRB = 0x05;
EIMSK = 0x30;
EIFR  = 0x30;


	// pull-up resistors on interrupt pins
	PORTE |= (1 << PE4) | (1 << PE5);

#else
// External Interrupt(s) initialization
// INT0: On
// INT0 Mode: Falling Edge
// INT1: On
// INT1 Mode: Falling Edge
// INT2: Off
// INT3: Off
// INT4: Off
// INT5: Off
// INT6: Off
// INT7: Off
EICRA=0x0A;
EICRB=0x00;
EIMSK=0x03;
EIFR=0x03;

	// pull-up resistors on interrupt pins
	PORTD |= (1 << PD1) | (1 << PD0);

#endif

#elif defined (__AVR_ATmega32__) || defined (__AVR_ATmega16__)

#if ENCODER_MULTICLICK
// External Interrupt(s) initialization
// INT0: On
// INT0 Mode: Any change
// INT1: On
// INT1 Mode: Any change
// INT2: Off
GICR|=0xC0;
MCUCR=0x05;
MCUCSR=0x00;
GIFR=0xC0;

	// pull-up resistors on interrupt pins
	PORTD |= (1 << PD2) | (1 << PD3);

#else
	// External Interrupt(s) initialization
	// INT0: On
	// INT0 Mode: Falling Edge
	// INT1: On
	// INT1 Mode: Falling Edge
	// INT2: Off
	GICR|=0xC0;
	MCUCR=0x0A;
	MCUCSR=0x00;
	GIFR=0xC0;

	// pull-up resistors on interrupt pins
	PORTD |= (1 << PD2) | (1 << PD3);

#endif

#endif

#if ENCODER_MULTICLICK
	{
#if defined (__AVR_ATmega128__)

	old_val = (PINE & ((1 << PE5) | (1 << PE4))) >> PE4;

#elif defined (__AVR_ATmega32__) || defined (__AVR_ATmega16__)
	old_val = (PIND & ((1 << PD3) | (1 << PD2))) >> PD2;
#endif
	}
#endif
}

// Timer 0 overflow interrupt service routine
// 7.813 kHz / 256 = approx 30 Hz
//       
//    .
ISR(TIMER0_OVF_vect)
{
	enc_spool();
	kbd_spool();
}

void timer_initialize(void)
{


#if defined (__AVR_ATmega128__)
// Timer/Counter 0 initialization
// Clock source: System Clock
// Clock value: 7.813 kHz
// Mode: Normal top=FFh
// OC0 output: Disconnected
TCCR0=0x07;		// in code vision avr error 
TCNT0=0x00;
OCR0=0x00;
TIMSK |= 0x01;
#elif defined (__AVR_ATmega32__) || defined (__AVR_ATmega16__)
// Timer/Counter 0 initialization
// Clock source: System Clock
// Clock value: 7.813 kHz
// Mode: Normal top=FFh
// OC0 output: Disconnected
TCCR0=0x07;	// in code vision avr error 
TCNT0=0x00;
OCR0=0x00;
TIMSK |= 0x01;
#endif

}

