#include <avr/io.h>

#include <avr/interrupt.h>
#include "keyboard.h"

#if defined (__AVR_ATmega128__)

#define KBD_TARGET_PORT PORTF
#define KBD_TARGET_DDR DDRF
#define KBD_TARGET_PIN PINF

#elif defined (__AVR_ATmega32__) || defined (__AVR_ATmega16__)

#define KBD_TARGET_PORT PORTA
#define KBD_TARGET_DDR DDRA
#define KBD_TARGET_PIN PINA

#endif


#if 0
///////adc

#define FIRST_ADC_INPUT 0
#define LAST_ADC_INPUT 7
static volatile unsigned char adc_data[LAST_ADC_INPUT-FIRST_ADC_INPUT+1];
#define ADC_VREF_TYPE 0x60

// ADC interrupt service routine
// with auto input scanning

static volatile unsigned char input_index;

ISR(ADC_vect)
{
	// Read the 8 most significant bits
	// of the AD conversion result
	adc_data[input_index]=ADCH;
	// Select next ADC input
	if (++input_index > (LAST_ADC_INPUT-FIRST_ADC_INPUT))
	   input_index=0;
	ADMUX=(FIRST_ADC_INPUT|ADC_VREF_TYPE)+input_index;
	// Start the AD conversion
	ADCSRA|=0x40;
}

void adc_initialize(void)
{
	input_index = 0;
	// ADC initialization
	// ADC Clock frequency: 62,500 kHz
	// ADC Voltage Reference: AVCC pin
	// Only the 8 most significant bits of
	// the AD conversion result are used
	ADMUX=FIRST_ADC_INPUT|ADC_VREF_TYPE;
	ADCSRA=0xCF;
	memset(adc_data, 0xff, sizeof adc_data / sizeof adc_data [0]);

}
#endif

static uint8_t 
keyboard_ged_pressed_key(uint8_t * scancode)
{
	uint8_t i;
	const uint8_t v = ~ KBD_TARGET_PIN;

	static const uint8_t codes [] =
	{
		KBD_CODE_B1, KBD_CODE_B2, KBD_CODE_B3, KBD_CODE_B4, 
		KBD_CODE_B5, KBD_CODE_B6, KBD_CODE_B7
	};

	static const uint8_t masks [] =
	{
		0x80, 0x40, 0x20, 0x10, 0x08, 0x04, 0x02
	};

	for (i = 0; i < (sizeof masks / sizeof masks [0]); ++ i)
	{
		if ((v & masks [i]) != 0)
		//if (adc_data [i] < 128)
		{
			* scancode = codes [i];
			return 1;
		}
	}
	return 0;
}


#define MAX_KBD_RELEASE_DELAY 2 //   ()   
#define MAX_KBD_PRESS_DELAY_LONG 20 //     

/* keyboard functions */
static uint8_t kbd_last;	/*  - (   ) */
static uint8_t kbd_press;	/*     */
static uint8_t kbd_release;		/*    -    */

/*  -   0   .
 *   ,  -    0x80
 */
static uint8_t
kbd_scan_local(uint8_t * key)
{
	uint8_t v;

	if (keyboard_ged_pressed_key(& v))
	{
		kbd_last = v;
		kbd_release = MAX_KBD_RELEASE_DELAY;
		if (kbd_press == MAX_KBD_PRESS_DELAY_LONG)
			return 0;	// lond_press symbol already returned
		if (kbd_press < MAX_KBD_PRESS_DELAY_LONG)
		{
			if (++ kbd_press == MAX_KBD_PRESS_DELAY_LONG)
			{	* key = (v | KBD_LONGPRESS); // lond_press symbol
				return 1;
			}
		}
		return 0x00;

	}
	else if (kbd_release != 0)
	{
		// keyboard keys released, time is not expire.
		if (-- kbd_release == 0)
		{
			// time is expire
			if (kbd_press < MAX_KBD_PRESS_DELAY_LONG)
			{
				* key = kbd_last;	
				kbd_press = 0;
				return 1;
			}
			else
			{
				kbd_press = 0;
				return 0;
			}
		}
		return 0;

	}
	else
		return 0;
}

static uint8_t kbd_scancode;
static uint8_t kbd_repeat;

// Called from Timer 0 overflow interrupt service routine
// 7.813 kHz / 256 = approx 30 Hz
void
kbd_spool(void)
{
	uint8_t code;
	if (kbd_scan_local(& code) != 0)
	{
		if (kbd_scancode == code)
			++ kbd_repeat;
		else
		{
			kbd_scancode = code;
			kbd_repeat = 1;
		}
	}
}
/*  -   0   .
 *   ,  -    0x80
 */
uint8_t kbd_scan(uint8_t * key, uint8_t * repeat)
{
	uint8_t r;
	cli();
	if (kbd_scancode != 0)
	{
		* key = kbd_scancode;
		* repeat = kbd_repeat;
		kbd_scancode = 0;
		r = 1;
	}
	else
	{
		r = 0;
	}
	sei();
	return r;
}


/*      */
/*  - -     */
void kbd_initialize(uint8_t * setupreq)
{
	uint8_t scancode;

	//adc_initialize();

	kbd_press = 0;
	kbd_release = 0;

	KBD_TARGET_PORT |= KBD_MASK;	// tie up inputs
	KBD_TARGET_DDR &= ~ KBD_MASK;	// define these bits as inputs

	kbd_scancode = 0;


	* setupreq = keyboard_ged_pressed_key(& scancode) && (scancode == KBD_CODE_B1);
}

