//
// mt-10t7.c


#include <stdlib.h>

/* Pin manipulation functioms */
#include <avr/io.h>
//#include <avr/pgmspace.h>
#include <util/delay.h>

#include "mt-10t7.h"

#if defined (__AVR_ATmega128__)

#define LCD_TARGET_PORT PORTA
#define LCD_DIRECTION_PORT DDRA
#define LCD_INPUT_PORT PINA

#elif defined (__AVR_ATmega32__) || defined (__AVR_ATmega16__)

#define LCD_TARGET_PORT PORTB
#define LCD_DIRECTION_PORT DDRB
#define LCD_INPUT_PORT PINB

#endif

#define STROBE_BIT 0x08
#define WRITEE_BIT 0x04
#define ADDRES_BIT 0x02

#define DATAS_BITS 0xf0
#define BUSYFLAG_BIT 0x80
//
// LCD melt-10t7 connections
//
//	PORTx0:	DB0 (pin 7)
//	PORTx1:	DB1 (pin 6)
//	PORTx2:	DB2 (pin 5)
//	PORTx3:	DB3 (pin 4)
//	PORTx4: A0 (pin 1) - '0' - addr, '1' - data
//  PORTx5: ~WR (pin 2) - '0' - write strob	
//  PORTx6:	
//  PORTx7:	

// LCD 1602 connections
//
//  PORTx0:	
//	PORTx1: A0 (pin 1) - '0' - addr, '1' - data
//  PORTx2: ~WR (pin 2) - '0' - write strob	
//  PORTx3:	
//	PORTx4:	DB0 (pin 7)
//	PORTx5:	DB1 (pin 6)
//	PORTx6:	DB2 (pin 5)
//	PORTx7:	DB3 (pin 4)

void mt10t7_wrpair(
	unsigned char addr, 
	unsigned char data);


/*
	DB0- g, DB1- e, DB2- d, DB3- a, DB4- h, DB5- b, DB6- c, DB7- f
*/
#define SEG_A 0x08
#define SEG_B 0x20
#define SEG_C 0x40
#define SEG_D 0x04
#define SEG_E 0x02
#define SEG_F 0x80
#define SEG_G 0x01
#define SEG_H 0x10

#if 0
static const unsigned char ten2led [] =
{
	0x00 | SEG_A | SEG_B | SEG_C | SEG_D | SEG_E | SEG_F,
	0x00 | SEG_B | SEG_C,
	0x00 | SEG_A | SEG_B | SEG_G | SEG_E | SEG_D,
	0x00 | SEG_A | SEG_B | SEG_G | SEG_C | SEG_D,
	0x00 | SEG_F | SEG_G | SEG_B | SEG_C,
	0x00 | SEG_A | SEG_F | SEG_G | SEG_D | SEG_C,
	0x00 | SEG_A | SEG_C | SEG_D | SEG_E | SEG_F | SEG_G,
	0x00 | SEG_A | SEG_B | SEG_C | SEG_F,
	0x00 | SEG_A | SEG_B | SEG_C | SEG_D | SEG_E | SEG_F | SEG_G,
	0x00 | SEG_A | SEG_B | SEG_C | SEG_D | SEG_F | SEG_G,
};


static const unsigned char mode2led [] =
{
	0x00 | SEG_C | SEG_D | SEG_E,	// small 'u' - USB
	0x00 | SEG_D | SEG_E,			// small 'l' - LSB
	0x00 | SEG_D | SEG_E | SEG_G,	// small 'c' - CW
};
#endif





#if 0
// ADDRES_BIT is set
void mt10t7_wrdata(
	unsigned char data)
{
	unsigned char hibits = (LCD_TARGET_PORT & ~ 0x1f);

	LCD_TARGET_PORT = hibits | (data & 0x0f) | ADDRES_BIT;

	LCD_TARGET_PORT &= ~STROBE_BIT; 	// activate ~wr signal
	LCD_TARGET_PORT |= STROBE_BIT; 	// deactivate ~wr signal

	LCD_TARGET_PORT = hibits | ((data >> 4) & 0x0f) | ADDRES_BIT;

	LCD_TARGET_PORT &= ~STROBE_BIT; 	// activate ~wr signal
	LCD_TARGET_PORT |= STROBE_BIT; 	// deactivate ~wr signal
}


void mt10t7_wraddr(
	unsigned char addr)
{
	unsigned char hibits = (LCD_TARGET_PORT & ~ 0x1f);
	// set address
	LCD_TARGET_PORT = hibits | (addr & 0x0f);

	LCD_TARGET_PORT &= ~STROBE_BIT; 	// activate ~wr signal
	LCD_TARGET_PORT |= STROBE_BIT; 	// deactivate ~wr signal
}


void mt10t7_wrpair(
	unsigned char addr, 
	unsigned char data)
{
	mt10t7_wraddr(addr);
	mt10t7_wrdata(data);
}


void mt10t7_init(void)
{
	unsigned char i;

	LCD_DIRECTION_PORT = 0xff;
	LCD_TARGET_PORT |= STROBE_BIT;	// deactivate write pin

	mt10t7_wrpair(0x0f, 0x11);	// unlock bus

	for (i = 0; i < 10; ++ i)
	{
		mt10t7_wrpair(i, 0x00);		// clear display
	}
}

void mt10t7_dispfreq(unsigned long freq, unsigned char mode, unsigned char special)
{
#define NDIGITS 7
	static const unsigned long vals [] =
	{
		10000000,
		1000000,
		100000,
		10000,
		1000,
		100,
		10,
		1,
	};
	static const unsigned char commas [] =
	{
		0x00, SEG_H, 0x00, 0x00, SEG_H, 0x00, 0x00, 0x00
	};

	unsigned  char i = 0;
	mt10t7_wraddr(0x00);
	
	for (i = 0; i < NDIGITS; ++ i)
	{
		ldiv_t res = ldiv(freq, vals [i]);
		if (i == 0 && res.quot == 0)
			mt10t7_wrdata(0x00 | commas [i]);	// supress zero
		else
			mt10t7_wrpair(i, ten2led [res.quot] | commas [i]);
		freq = res.rem;
	}

	mt10t7_wrpair(0x09, mode2led [mode] | (special ? SEG_A : 0x00));
}


#endif

static void ws1602_send4_high(unsigned char v)
{
	LCD_TARGET_PORT = (LCD_TARGET_PORT & ~ DATAS_BITS) | (v & DATAS_BITS);
	_delay_us(50);
}
static void ws1602_send4_low(unsigned char v)
{
	LCD_TARGET_PORT = (LCD_TARGET_PORT & ~ DATAS_BITS) | ((v << 4) & DATAS_BITS);
	_delay_us(50);
}

static void ws1602_pulse_write(void)
{
	LCD_TARGET_PORT |= STROBE_BIT; 	// activate EN signal
	_delay_us(50);
	LCD_TARGET_PORT &= ~STROBE_BIT; // deactivate EN signal
	_delay_us(50);
}

static void ws1602_to_read(unsigned char as)
{
	if (as)
		LCD_TARGET_PORT |= ADDRES_BIT;
	else
		LCD_TARGET_PORT &= ~ ADDRES_BIT;

	_delay_us(50);
	LCD_DIRECTION_PORT &= ~ DATAS_BITS;
	LCD_TARGET_PORT |= WRITEE_BIT;
	_delay_us(50);
}

static void ws1602_to_write(unsigned char as)
{
	if (as)
		LCD_TARGET_PORT |= ADDRES_BIT;
	else
		LCD_TARGET_PORT &= ~ ADDRES_BIT;

	_delay_us(50);
	LCD_TARGET_PORT &= ~WRITEE_BIT;
	LCD_DIRECTION_PORT |= DATAS_BITS;
	_delay_us(50);
}

static void ws1602_wait_ready(void)
{
	uint8_t n;

	ws1602_to_read(0);
	for (n = 50; -- n;)
	{
		unsigned char stsh;
		LCD_TARGET_PORT |= STROBE_BIT; 	// activate EN signal
		_delay_us(50);
		stsh = LCD_INPUT_PORT;
		LCD_TARGET_PORT &= ~STROBE_BIT; 	// deactivate EN signal
		_delay_us(50);

		if ((stsh & BUSYFLAG_BIT) == 0)
			break;
	}
}

// rs = 1
void ws1602_wrdata(
	unsigned char v)
{
	ws1602_wait_ready();

	ws1602_to_write(1);
	ws1602_send4_high(v);
	ws1602_pulse_write();

	ws1602_wait_ready();

	ws1602_to_write(1);
	ws1602_send4_low(v);
	ws1602_pulse_write();
}

// rs = 0
void ws1602_wrcmd(
	unsigned char v)
{

	ws1602_wait_ready();

	ws1602_to_write(0);
	ws1602_send4_high(v);
	ws1602_pulse_write();

	ws1602_wait_ready();

	ws1602_to_write(0);
	ws1602_send4_low(v);
	ws1602_pulse_write();
}

// rs = 0
void ws1602_wrcmd_nowait(
	unsigned char v)
{
	_delay_ms(20);

	ws1602_to_write(0);
	ws1602_send4_high(v);
	ws1602_pulse_write();

	_delay_ms(20);

	ws1602_to_write(0);
	ws1602_send4_low(v);
	ws1602_pulse_write();

}

void ws1602_initialize(void)
{
	LCD_TARGET_PORT = 0x00;
	LCD_DIRECTION_PORT |= // all control bits are output
		(STROBE_BIT | WRITEE_BIT | ADDRES_BIT);

	LCD_TARGET_PORT &= ~STROBE_BIT;	// deactivate enable pin

	ws1602_to_write(0);

	// switch interface to 4-bit wide mode
	ws1602_wrcmd_nowait(0x33);
	ws1602_wrcmd_nowait(0x32);

	ws1602_wrcmd(0x28);		// two lines

	ws1602_wrcmd(0x0c);       // .LCD ,  .,  . .
	ws1602_wrcmd(0x06);		//    

	ws1602_wrdata('O');		// test string - in memory of Basic
	ws1602_wrdata('K');
	ws1602_wrdata('>');
}

/* x, y - zero-based positions */
void ws1602_gotoxy(uint8_t x, uint8_t y)
{
	ws1602_wrcmd(0x80 | (y << 6) | x);	//   0 pos  
}

void ws1602_string(const char * s)
{
	char c;
	while ((c = (* s ++)) != '\0')
		ws1602_wrdata(c);
}

void lcd_menu_value(
	unsigned long freq,
	unsigned char width,	// full width
	unsigned char comma,		// comma position (from right, inside width)
	unsigned char rj		// right truncated
	)
{
	//PROGMEM 
	static const unsigned long vals [] =
	{
		10000000,
		1000000,
		100000,
		10000,
		1000,
		100,
		10,
		1,
	};
	unsigned  char j = (sizeof vals /sizeof vals [0]) - rj;
	unsigned  char i = (j - width);
	unsigned  char z = 1;	// only zeroes

	ws1602_gotoxy(0, 0);		//     

	for (; i < j; ++ i)
	{
		//ldiv_t res = ldiv(freq, pgm_read_dword(& vals [i]));
		ldiv_t res = ldiv(freq, vals [i]);
		if (comma == (j - i))
			ws1602_wrdata('.');

		if (z == 1 && (i + 1) < j && res.quot == 0)
			ws1602_wrdata(' ');	// supress zero
		else
		{
			z = 0;
			ws1602_wrdata('0' + res.quot);
		}
		freq = res.rem;

	}
}

void lcd_menu_name(
	unsigned char paramno,	/* zero-based parameter index */
	const char * paramname	/* parameter name string */
	)
{
	ws1602_gotoxy(0, 1);				//   8 pos 1-st string
	ws1602_wrdata('F');
	ws1602_wrdata('0' + paramno / 10);
	ws1602_wrdata('0' + paramno % 10);
	ws1602_wrdata(' ');
	ws1602_string(paramname);
}

void ws1602_dispfreq(unsigned long freq)
{
	lcd_menu_value(freq, 7, 2, 1);
}

void ws1602_dispmode(
	const char * mode, 
	unsigned char noaccelerate, 
	unsigned char att, 
	unsigned char lock, 
	const char * agcmode, 
	const char * bw)
{
	ws1602_gotoxy(8, 0);				//   8 pos 1-st string
	ws1602_string(lock ? "*" : " ");
	ws1602_string(mode);
	ws1602_gotoxy(13, 0);				//   12 pos 1-st string
	ws1602_string(bw);

	ws1602_gotoxy(0, 1);
	ws1602_string(att ? "Att " : "    ");

	ws1602_gotoxy(8, 1);
	ws1602_string(noaccelerate ? "X" : " ");

	ws1602_gotoxy(12, 1);
	ws1602_string(agcmode);
}

void lcd_clear(void)
{
	ws1602_wrcmd(0x01);		//  
	_delay_ms(5);
}

void ws1602_update(void)	// one step updating information
{


}

void ws1602_message(const char * s)
{
	ws1602_gotoxy(0, 0);
	ws1602_string(s);
}
