/* spifuncs.c */
#include <stdlib.h>
#include <assert.h>

/* Pin manipulation functions */
#include <avr/io.h>
#include <util/delay.h>
//#include <avr/pgmspace.h>

#include "board.h"
#include "product.h"
#include "synthcalcs.h"
#include "spifuncs.h"

/* 
 *    -      
 */

void prog_bit(
	unsigned char target,	/* addressing to chip */
	uint8_t bitv	/* zero or non-zero values */
	)
{
	SDO_SET(bitv);	//   
	SCLK_CLR();	// latch to chips
	SCLK_SET();	// latch to 74HC595!
}


/* switch off all chip selects and data enable */

void 
prog_select_init(void)
{
#if defined (__AVR_ATmega128__)
	PORTC = 0x00;	// synthesizer data port initial state
	DDRC = 0xFF;	// synthesizer data port

	PORTD |= 0x80;	// inactive nvram chipselect
	DDRD |= 0x80;	// nvram chipselect
	DDRD &= ~ 0x40;	// nvram data in define these bits as inputs

#elif defined (__AVR_ATmega32__) || defined (__AVR_ATmega16__)
	PORTC = 0x00;
	DDRC = 0xFF;

	PORTD |= 0x02;	// inactive nvram chipselect
	DDRD |= 0x02;	// nvram chipselect
	DDRD &= ~ 0x01;	// nvram data in define these bits as inputs

#endif

	lpt_writeb(0x00);
	fclk_off();	// chip select inactive
	SCLK_SET();	// initial state of SCLK - logical "1"
}

/* 
 *    -  
 */

void prog_select(
	unsigned char target,	/* addressing to chip */
	fastbool_t select	/* zero - done sending data, non-zero - start sending data */
	)
{
	if (target == targetnvram)
	{
#if defined (__AVR_ATmega128__)

		if (select)
		{
			PORTD &= ~ (1U << PD7);	// PD7 = 1: nvram cs off	
		}
		else
		{
			PORTD |= (1U << PD7);	// PD7 = 1: nvram cs off	
		}


#elif defined (__AVR_ATmega32__) || defined (__AVR_ATmega16__)

		if (select)
		{
			PORTD &= ~ (1U << PD1);	// PD1 = 1: nvram cs off	
		}
		else
		{
			PORTD |= (1U << PD1);	// PD1 = 1: nvram cs off	
		}

#endif
		return;
	}

	if (select)
	{
		SCLK_SET();	// initial state of SCLK - logical "1"
	//::Sleep(10);
		lpt_and_writeb(target << 2, 0x07 << 2);
	//::Sleep(10);
		fclk_on();	// chip select active
	//\::Sleep(10);
	}
	else
	{
		fclk_off();	// chip select inactive
	//::Sleep(10);
		SCLK_SET();	// initial state of SCLK - logical "1"
	//::Sleep(10);
	}
}



uint8_t phase_getbit(
	const phase_t * v,
	fastcount_t i		/* bit position, LSB = 0 */
	);


/* send bits (starting from MSB) */
void prog_phbits(
	unsigned char target,	/* addressing to chip */
	const phase_t * val,
	fastcount_t i,				/* left of 1-st bit to set */
	fastcount_t n				/* number of bits to send */
	)
{
	for (; n --; )
	{
		prog_bit(target, phase_getbit(val, -- i));
	}
}

void prog_val(
	unsigned char target,	/* addressing to chip */
	vt_t value,
	fastcount_t n				/* number of bits to send */
	)
{
	fastcount_t i;
	vt_t mask = ((vt_t) 1) << (n - 1);
	//for (i = n; -- i < n && i >= 0; )
	for (i = n; -- i < n; )
	{
		prog_bit(target, (vt_t) ((value & mask) != 0));
		mask >>= 1;
	}
}


uint8_t phase_getbit(
	const phase_t * v,
	fastcount_t i		/* bit position, LSB = 0 */
	)
{
	static const uint8_t mask [8] = { 0x01, 0x02, 0x04, 0x08, 0x10, 0x20, 0x40, 0x80 };
	return (((const uint8_t *) v) [i >> 3] & mask [i & 07]);

	//div_t res = div(i, 8);
	//return (((const unsigned char *) v) [res.quot] & mask [res.rem]) != 0;
	//static div_t res = div(i, 8);
}

