/* 	
	   
	    

	  - Genadi V. Zawidowski
	mgs2001@pochtamt.ru
	Amateur radio call: UA1ARN
*/

#include <stdint.h>
#include <stdlib.h>
#include <stddef.h>


#include "product.h"
#include "synthcalcs.h"
#include "board.h"
#include "mt-10t7.h"
#include "keyboard.h"
#include "encoder.h"

//#include <avr/eeprom.h>
#include <avr/interrupt.h>
#include <avr/io.h>
//#include <avr/pgmspace.h>
#include <util/delay.h>

#define WITHMENU 1	/*      -    */

/*  ,     (  ),
       (        ).
*/
#define BANDPAD (25000 / 1000)

static struct bandrange {
	uint16_t bottom1k, top1k;
	uint16_t init1k;

} const bandsmap [] =
{
	{ 1800 - BANDPAD, 2000 + BANDPAD, 1800, },		/* 0 */
	{ 3500 - BANDPAD, 4000 + BANDPAD, 3500, },		/* 1 */
	{ 7000 - BANDPAD, 7300 + BANDPAD, 7000, },		/* 2 */
	{ 10100 - BANDPAD, 10150 + BANDPAD, 10100, },		/* 3 */
	{ 14000 - BANDPAD, 14350 + BANDPAD, 14000, },		/* 4 */
	{ 18068 - BANDPAD, 18168 + BANDPAD, 18068, },		/* 5 */
	{ 21000 - BANDPAD, 21450 + BANDPAD, 21000, },		/* 6 */
	{ 24890 - BANDPAD, 24990 + BANDPAD, 24890, },		/* 7 */
	{ 28000 - BANDPAD, 29700 + BANDPAD, 28000, },		/* 8 */
	{ 50000 - BANDPAD, 54000 + BANDPAD, 50000, },		/* 9 */

	{ 100, 30000, 4997, },		/* HBANDS_COUNT should equal to this index */
	{ 30000, 60000, 39000, },
};

#define XBANDS_COUNT 2
#define HBANDS_COUNT ((sizeof bandsmap / sizeof bandsmap [0]) - XBANDS_COUNT)



typedef struct filter_tag
{
	uint8_t modes;	/* modefamily bits - code of filter for RX board control */
	char name [4];	/* name of filter	*/
	uint16_t low10;		/* frequency of low edge (for ssb detector) - in tens of hertz */
	uint16_t high10;		/* frequency of high edge (for ssb detector)  - in tens of hertz*/
	uint8_t code;	/* code of filter for RX board control */
} filter_t;

/*    */
enum {
	MODE_CW,	/* cw & cwr */
	MODE_SSB,	/* usb & lsb */
	MODE_AM,
	MODE_FM,
	MODE_DRM,
	MODE_CWZ,
	//
	MODE_COUNT
};

/*      .
 *  -    +  +  
 */
enum { 
	SUBMODE_USB, SUBMODE_LSB, 
	SUBMODE_CW, SUBMODE_CWR, SUBMODE_CWZ, 
	SUBMODE_AM,
	SUBMODE_DRM,
	SUBMODE_FM,
	//
	SUBMODE_COUNT
};


/* "" , 
 *         "",
 *          "".
 */
#if defined(MODEL_4Z5KY)
static const unsigned char modes [][4] =
{
	{ 2, SUBMODE_USB, SUBMODE_LSB, },
	{ 2, SUBMODE_AM6, SUBMODE_AMW },
};
#else
static const unsigned char modes [][4] =
{
	{ 2, SUBMODE_LSB, SUBMODE_USB, },
	{ 2, SUBMODE_CWR, SUBMODE_CW },
	{ 3, SUBMODE_AM, SUBMODE_DRM, SUBMODE_CWZ, },
	{ 1, SUBMODE_FM, },
};

#endif

#define MODEROW_COUNT (sizeof modes / sizeof modes [0])

/*      .
      -  0   0:0
*/
static uint8_t
locatemode(
	uint8_t submode,		/*   */
	uint8_t * xrow,		/*   */
	uint8_t * xcol)
{
	uint8_t row, col;

	for (row = 0; row < MODEROW_COUNT; ++ row)
	{	const uint8_t n = modes [row][0];
		for (col = 0; col < n; ++ col)
		{
			if (modes [row][col + 1] == submode)
			{
				* xrow = row;
				* xcol = col;
				return 1;
			}
		}
	}
	* xrow = 0;
	* xcol = 0;
	return 0;
}

/*    " "  . */
static uint8_t
getdefaultmode(
	uint32_t freq	/*  (  ) */
	)
{
	return freq < 10000000 ? SUBMODE_LSB : SUBMODE_USB;
}

/*  -     .
 	 ""  - CW&CWR, LSB&USB.
	*/
struct modefmly
{
	uint8_t agc;	/*      */
	uint8_t filter;	/*       */
	uint16_t step;	/*      */

};
/*  -     .
 ,     */
struct bandinfo
{
	uint32_t freq;
	uint8_t moderow;
	uint8_t modecol [MODEROW_COUNT];
	uint8_t att;
};

#define MBANDS_COUNT	25	/*     */
#define MBANDS_BASE (HBANDS_COUNT + XBANDS_COUNT)	/*      */
/*  -     .
   bitfields  ,     -   
	    .
*/
struct nvmap
{
	uint32_t afreq;		/*   ,     */
	uint8_t lock;			/*   */
	uint8_t menu;			/*   ,    */
	uint8_t mband;		/*   ,    */
	uint8_t mfixed;		/* -0,      */
	struct modefmly modes [MODE_COUNT];
	struct bandinfo bands [HBANDS_COUNT + XBANDS_COUNT + MBANDS_COUNT];
	//uint8_t ssbstep;
	//uint8_t amstep;
	uint16_t cwpitch;
	uint16_t usbe3p1;
	uint16_t lsbe3p1;
	uint16_t usbe6p0;
	uint16_t lsbe6p0;
	uint16_t noaccelerate;	/*    */
};


/* ,     FRAM */
#define RMT_FREQ_BASE offsetof(struct nvmap, afreq)			/*  ,    (4 ) */
#define RMT_MENU_BASE	offsetof(struct nvmap, menu)		/*  -   ,    */
#define RMT_LOCKMODE_BASE offsetof(struct nvmap, lock)		/*    */
#define RMT_MFIXED_BASE offsetof(struct nvmap, mfixed)		/*    */


#define RMT_AGC_BASE(i)	offsetof(struct nvmap, modes[i].agc)
#define RMT_FILTER_BASE(i)	offsetof(struct nvmap, modes[i].filter)
#define RMT_STEP_BASE(i)	offsetof(struct nvmap, modes[i].step)

#define RMT_BFREQ_BASE(i) offsetof(struct nvmap, bands[i].freq)			/*  ,    (4 ) */
#define RMT_ATT_BASE(i) offsetof(struct nvmap, bands[i].att)			/*    (1 ) */
#define RMT_MODEROW_BASE(i)	offsetof(struct nvmap, bands[i].moderow)			/*     . */
#define RMT_MODECOL_BASE(i)	offsetof(struct nvmap, bands[i].modecol)			/*      . */

//	MODE_CW,	/* cw & cwr */
//	MODE_SSB,	/* usb & lsb */
//	MODE_AM.
//	MODE_FM,
//	MODE_DRM.


/*  .   -    
*/
typedef struct submodeprops_tag
{
	uint8_t lsb : 1;		/*        	 (CWR, LSB) */
	uint8_t detector : 2;	/*   */
	uint8_t mode : 3;			/*    */
	const char name [4];
} submodeprops_t;

static const submodeprops_t gsubmodes [SUBMODE_COUNT] =
{
	{	
		0x00,			/* lsb flag = 0 */
		0x00, 		/* ssb detector used */
		MODE_SSB, 	/*    */
		"USB", 
	},
	{	
		0x01,			/* lsb flag = 0 */
		0x00, 		/* ssb detector used */
		MODE_SSB, 	/*    */
		"LSB", 
	},
	{	
		0x00,			/* lsb flag = 0 */
		0x00, 		/* ssb detector used */
		MODE_CW, 	/*    */
		"CW ", 
	},
	{	
		0x01,			/* lsb flag = 0 */
		0x00, 		/* ssb detector used */
		MODE_CW, 	/*    */
		"CWR", 
	},
	{	
		0x00,			/* lsb flag = 0 */
		0x00, 		/* ssb detector used */
		MODE_CWZ, 	/*    */
	  	"CWZ", 
	},
	{	
		0x00,			/* lsb flag = 0 */
		0x01, 		/* AM detector used */
		MODE_AM, 	/*    */
		"AM ",
	},
	{	
		0x00,			/* lsb flag = 0 */
		0x02, 		/* ssb detector used, mute audio */
		MODE_DRM, 	/*    */
		"DRM", 
	},
	{	
		0x00,			/* lsb flag = 0 */
		0x03, 		/* FM detector used */
		MODE_FM, 	/*    */
		"FM ",
	},

};


#if defined (IF3_MODEL) && (IF3_MODEL == IF3_TYPE_455)
	// filter codes:
	// 0x06 - fil3 3.1 kHz
	// 0x01 - fil2 2.7 kHz
	// 0x00 - fil0 9.0 kHz
	// 0x02 - fil1 6.0 kHz
	// 0x03 - fil5 0.5 kHz
	// 0x04 - fil2 15 kHz (bypass)
	// 0x05 - unused
	// 0x07 - unused
static filter_t fi_0p3 =
{
	(1U << MODE_CW) | (1U << MODE_CWZ),
	"0.3",		/*     */
	45500 - (30/2),	/*      */
	45500 + (30/2),	/*      */
	0x03					/*        */
};

static filter_t fi_0p5 =
{
	(1U << MODE_CW) | (1U << MODE_CWZ),
	"0.5",		/*     */
	45500 - (50/2),	/*      */
	45500 + (50/2),	/*      */
	0x03					/*        */
};

static filter_t fi_1p8 =
{
	(1U << MODE_CW) | (1U << MODE_CWZ) | (1U << MODE_SSB),
	"1.8",		/*     */
	45500 - (180/2),	/*      */
	45500 + (180/2),	/*      */
	0x01					/*        */
};

static filter_t fi_2p75 =
{
	(1U << MODE_CW) | (1U << MODE_CWZ) | (1U << MODE_SSB) | (1U << MODE_AM),
	"2.7",		/*     */
	45500 - (275/2),	/*      */
	45500 + (275/2),	/*      */
	0x01					/*        */
};

static filter_t fi_3p1 =
{
	(1U << MODE_CW) | (1U << MODE_CWZ) | (1U << MODE_SSB) | (1U << MODE_AM),
	"3.1",	/*     */
	45500 - (370/2),	/*      */
	45500 + (370/2),	/*      */
	0x06				/*        */
};

static filter_t fi_6p0 =
{
	(1U << MODE_CWZ) | (1U << MODE_SSB) | (1U << MODE_AM) | (1U << MODE_FM)  | (1U << MODE_DRM),
	"6.0",		/*     */
	45200,				/*      */
	45800,				/*      */
	0x02				/*        */
};


static filter_t fi_9p0 =
{
	(1U << MODE_CWZ) | (1U << MODE_SSB) | (1U << MODE_AM) | (1U << MODE_FM)  | (1U << MODE_DRM),
	"9.0",		/*     */	
	45050,				/*      */
	45950,				/*      */
	0x00				/*        */
};

static const filter_t fi_15p0 =
{
	(1U << MODE_CWZ) | (1U << MODE_SSB) | (1U << MODE_AM) | (1U << MODE_FM)  | (1U << MODE_DRM),
	"15.",		/*     */
	44750,				/*      */
	46250,				/*      */
	0x04				/*        */
};

#endif // MODEL_80455

#if defined (IF3_MODEL) && (IF3_MODEL == IF3_TYPE_500)
	// filter codes:
	// 0x06 - fil3 3.1 kHz
	// 0x01 - fil2 2.7 kHz
	// 0x00 - fil0 9.0 kHz
	// 0x02 - fil1 6.0 kHz
	// 0x03 - fil5 0.5 kHz
	// 0x04 - fil2 15 kHz (bypass)
	// 0x05 - unused
	// 0x07 - unused

// 500-3-v
static filter_t fi_3p1 =
{
	(1U << MODE_CW) | (1U << MODE_CWZ) | (1U << MODE_SSB) | (1U << MODE_AM),
	"3.1",		/*     */
	50000,					/*      */
	50350,					/*      */
	0x06					/*        */
};
static const filter_t fi_2p75 =
{
	(1U << MODE_CW) | (1U << MODE_CWZ) | (1U << MODE_SSB) | (1U << MODE_AM),
	"2.7",		/*     */
	49680,					/*      */
	49845,					/*     */
	50010,					/*      */
	0x01					/*        */
};
static filter_t fi_0p5 =
{
	(1U << MODE_CW) | (1U << MODE_CWZ),
	"0.5",		/*     */
	49975,					/*      */
	50025,					/*      */
	0x03					/*        */
};
static const filter_t fi_15p0 =
{
	(1U << MODE_CWZ) | (1U << MODE_SSB) | (1U << MODE_AM) | (1U << MODE_FM)  | (1U << MODE_DRM),
	"15.",	/*     */
	49400,					/*      */
	50600,					/*      */
	0x04					/*        */
};
#endif

#if defined (IF3_MODEL) && (IF3_MODEL == IF3_TYPE_215)
static const filter_t fi_3p0 =
{
	(1U << MODE_CW) | (1U << MODE_CWZ) | (1U << MODE_SSB) | (1U << MODE_AM),
	"3.0",		/*     */
	213150,					/*      */
	216850,					/*      */
	0x00					/*        */
};
#endif	/* MODEL_35 */

#if defined (IF3_MODEL) && (IF3_MODEL == IF3_TYPE_8868)
const filter_t fi_3p1 =
{
	(1U << MODE_CW) | (1U << MODE_CWZ) | (1U << MODE_SSB) | (1U << MODE_AM),
	"3.1",
	8863000,
	8866000,
	0x00
};

const filter_t fi_0p5 =
{
	(1U << MODE_CW) | (1U << MODE_CWZ),
	"0.5",
	8863000,
	8866000,
	0x00
};

#endif

#if defined(MODEL_4Z5KY)
static const filter_t fi_6p0 =
{
	(1U << MODE_CW) | (1U << MODE_CWZ) | (1U << MODE_SSB) | (1U << MODE_AM) | (1U << MODE_FM)  | (1U << MODE_DRM),
	"6.0",		/*     */
	452000,				/*      */
	455000,				/*     */
	458000,				/*      */
	0x02				/*        */
};
#endif


/*      FRAM   */
uint8_t restore_i8(uint16_t addr)
{
	uint8_t v;
	nvram_read(addr, & v, sizeof v);
	return v;
}

/*      FRAM   */
void save_i8(uint16_t addr, uint8_t v)
{
	nvram_write(addr, & v, sizeof v);
}

/*      FRAM  16-  */
uint16_t restore_i16(uint16_t addr)
{
	uint16_t v;
	nvram_read(addr, & v, sizeof v);
	return v;
}

/*      FRAM  16-  */
void save_i16(uint16_t addr, uint16_t v)
{
	nvram_write(addr, & v, sizeof v);
}

/*      FRAM  32-  */
uint32_t restore_i32(uint16_t addr)
{
	uint32_t v;
	nvram_read(addr, & v, sizeof v);
	return v;
}

/*      FRAM  32-  */
void save_i32(uint16_t addr, uint32_t v)
{
	nvram_write(addr, & v, sizeof v);
}


/*      low..high  "" */
/*      */
unsigned char calc_next(unsigned char v, unsigned char low, unsigned char high)
{
	if (v < low || v >= high)
		return low;
	return v + 1;
}
/*      low..high  "" */
/*      */
unsigned char calc_prev(unsigned char v, unsigned char low, unsigned char high)
{
	if (v <= low || v > high)
		return high;
	return v - 1;
}

/*      */
/*      */
static unsigned char 
calc_next_list_i8(
	unsigned char v, 
	const uint8_t * list, 	/*   */
	uint8_t len)				/*   */
{
	uint8_t i;
	for (i = 0; i < len; ++ i)
		if (list [i] == v)
			break;
	if (i == len)	/*        -     */
		return list [0];

	i = calc_next(i, 0, len - 1);
	return list [i];
}

/*      */
/*      */
static unsigned char 
calc_next_list_i32(
	unsigned char v, 
	const uint32_t * list, 	/*   */
	uint8_t len)				/*   */
{
	uint8_t i;
	for (i = 0; i < len; ++ i)
		if (list [i] == v)
			break;
	if (i == len)	/*        -     */
		return list [0];

	i = calc_next(i, 0, len - 1);
	return list [i];
}


/*      ,    */
/* freq -  , step -  */
static uint32_t nextfreq(uint32_t oldfreq, uint32_t freq, uint32_t step, uint32_t top)
{
	ldiv_t r;

	r = ldiv(freq, step);
	if (r.rem != 0)
		freq = r.quot * step;

	if (oldfreq > freq)
		return top - step;
	if (freq >= top)
		return top - step;

	return freq;
}

/*      ,    */
/* freq -  , step -  */
static uint32_t prevfreq(uint32_t oldfreq, uint32_t freq, uint32_t step, uint32_t bottom)
{
	ldiv_t r;

	r = ldiv(freq, step);
	if (r.rem != 0)
		freq = (r.quot + 1) * step;

	if (oldfreq < freq)
		return bottom;
	if (freq < bottom)
		return bottom;

	return freq;
}

/*     
 */
enum { 
	AGCMODE_SLOW, 
	AGCMODE_MED, 
	AGCMODE_FAST, 
	AGCMODE_OFF, 
	//
	AGCMODE_COUNT
	};
/* ,      .
 *  ()     
 *      enum.
 */
static const char agcmode2string [AGCMODE_COUNT][5] =
{
	"Slow",	"Med ", "Fast", "AGC-", 
};

static unsigned char getagcoff(
	unsigned char mode,		/*    */
	unsigned char agcmode	/*    */
	)
{
	return (AGCMODE_OFF == agcmode) ? 0x01 : 0x00;
}
static unsigned char getagccode(
	unsigned char submode,		/* not used -    */
	unsigned char agcmode	/*    */
	)
{
	enum
	{
		AGC_CODE_SLOW = 0x07,
		AGC_CODE_MED  = 0x03,
		AGC_CODE_FAST = 0x02, //0x01,
		AGC_CODE_OFF = 0x00
	};
	switch (agcmode)
	{
	default:
	case AGCMODE_SLOW:
		return AGC_CODE_SLOW;

	case AGCMODE_FAST:
		return AGC_CODE_FAST;

	case AGCMODE_MED:
		return AGC_CODE_MED;

	case AGCMODE_OFF:
		return AGC_CODE_OFF;

	}
}


#if defined (IF3_MODEL) && (IF3_MODEL == IF3_TYPE_500)

/*  ,    */
static const filter_t * const gfa [] =
{
		& fi_0p3,
		& fi_0p5,
		//& fi_1p8,
		//& fi_2p75,
		& fi_3p1,
		//& fi_6p0,
		//& fi_9p0,
		& fi_15p0,
};

#endif

#if defined (IF3_MODEL) && (IF3_MODEL == IF3_TYPE_455)


/*  ,    */
static const filter_t * const gfa [] =
{
		//& fi_0p3,
		& fi_0p5,
		//& fi_1p8,
		//& fi_2p75,
		& fi_3p1,
		& fi_6p0,
		& fi_9p0,
		& fi_15p0,
};

#endif


#if defined (IF3_MODEL) && (IF3_MODEL == IF3_TYPE_8868)

/*  ,    */
static const filter_t * const gfa [] =
{
		& fi_0p5,
		& fi_3p1,
};

#endif

#if defined (IF3_MODEL) && (IF3_MODEL == IF3_TYPE_DCRX)

// 500-3-v
static filter_t fi_3p1 =
{
	(1U << SUBMODE_CW) | (1U << SUBMODE_CWR) | (1U << SUBMODE_CWZ) | (1U << SUBMODE_USB) | (1U << SUBMODE_LSB) | (1U << SUBMODE_AM) | (1U << SUBMODE_FM),
	"3.1",		/*     */
	0,					/*      */
	0,					/*     */
	0,					/*      */
	0x06					/*        */
};

static filter_t fi_0p5 =
{
	(1U << SUBMODE_CW) | (1U << SUBMODE_CWR) | (1U << SUBMODE_CWZ),
	"0.5",		/*     */
	0,/*      */
	0,				/*     */
	0,	/*      */
	0x03					/*        */
};


/*  ,    */
static const filter_t * const gfa [] =
{
		& fi_0p5,
		& fi_3p1,
};

#endif



/*   ,     */
static uint8_t getsuitable(
	uint8_t submode,
	uint8_t ix)
{
	const uint8_t ix0 = ix;
	const uint8_t mask = 1U << gsubmodes [submode].mode;
	const uint8_t n = sizeof gfa / sizeof gfa [0];	/*   */
	while ((gfa [ix]->modes & mask) == 0)
	{
		ix = calc_next(ix, 0, n - 1);
		if (ix == ix0)
			break;
	}
	return ix;
}

static uint8_t loadvfy8(
	uint16_t place, 
	uint8_t bottom, uint8_t top, uint8_t def)
{
	uint8_t v;

	// pre-chechk default value added for mode row switching with same column as default
	if (def >= top || def < bottom)
		def = bottom;
	v = restore_i8(place);
	if (v >= top || v < bottom)
	{
		v = def;
		save_i8(place, v);
	}
	return v;
}


static uint16_t loadvfy16(
	uint16_t place, 
	uint16_t bottom, uint16_t top, uint16_t def)
{
	uint16_t v;

	if (def >= top || def < bottom)
		def = bottom;
	v = restore_i16(place);
	if (v >= top || v < bottom)
	{
		v = def;
		save_i16(place, v);
	}
	return v;
}

static uint32_t loadvfy32(
	uint16_t place, 
	uint32_t bottom, uint32_t top, uint32_t def)
{
	uint32_t v;

	if (def >= top || def < bottom)
		def = bottom;
	v = restore_i32(place);
	if (v >= top || v < bottom)
	{
		v = def;
		save_i32(place, v);
	}
	return v;
}

static uint32_t loadvfy32freq(
	uint8_t band
	)
{
	const uint32_t lowfreq = bandsmap [band].bottom1k * 1000UL;
	const uint32_t topfreq = bandsmap [band].top1k * 1000UL;
	const uint32_t inifreq = bandsmap [band].init1k * 1000UL;
	return loadvfy32(RMT_BFREQ_BASE(band), lowfreq, topfreq, inifreq);
}

#define DEFAULT_CW_PITCH10 ((uint16_t) 70)	/*   ( -     ,    CW_PITCH) */
#define DRM_PITCH10 ((uint16_t) 1200)	/*  DRM */

/* 
 *    -   (  ). 
 *        cwpitch,  SSB -    
 */
static signed int gettone10(
	unsigned char submode,		/*    */
	int cwpitch10				/*    -   . */
	)
{
	switch (submode)
	{
	default:
	case SUBMODE_USB: 
	case SUBMODE_LSB: 
		return 0;

	case SUBMODE_CW: 
		return + cwpitch10;
	case SUBMODE_CWR: 
		return - cwpitch10;
	case SUBMODE_CWZ: 
		return 0;

	case SUBMODE_AM: 
		return 0;


	case SUBMODE_DRM:
		return 0;

	case SUBMODE_FM:
		return 0;
	}
}


/*        /    .
 *  AM  0
 */
static unsigned long getlo3(
	const filter_t * workfilter,
	unsigned char submode,		/*    */
	unsigned char forcalc, 	/*       ,     */
	int cwpitch10,				/*    -    */
	uint8_t if1inverse
	)
{
	switch (submode)
	{
	default:
		return ((uint32_t) workfilter->low10 + workfilter->high10) * 5UL;	/* 5     10 */

	case SUBMODE_USB: 
		return (if1inverse ? workfilter->low10 : workfilter->high10) * 10UL;

	case SUBMODE_LSB: 
		return (if1inverse ? workfilter->high10: workfilter->low10) * 10UL;

	case SUBMODE_CWZ: 
	case SUBMODE_CW: 
	case SUBMODE_CWR: 
		return forcalc ? 
			((uint32_t) workfilter->low10 + workfilter->high10) * 5UL 
			: 
			(((uint32_t) workfilter->low10 + workfilter->high10) / 2UL + gettone10(submode, cwpitch10))  * 10UL ;


	case SUBMODE_AM: 
	case SUBMODE_FM:
		return forcalc ? 
			((uint32_t) workfilter->low10 + workfilter->high10) * 5UL 
			:
			 0UL;

	case SUBMODE_DRM:
		return forcalc ?
			((uint32_t) workfilter->low10 + workfilter->high10) * 5UL 
			:
			(((uint32_t) workfilter->low10 + workfilter->high10) / 2UL + DRM_PITCH10) * 10UL ;

	}
}

/*  LO2     */
static unsigned long getlo2(
	unsigned char submode		/*    */
	)
{
	return LO2FREQ;
}

/*      .
 */
#define STEP_SSB	10 //50		/*   SSB (USB/LSB */
#define	STEP_AM		100		/*   AM */
#define STEP_DRM	5000	/*   DRM */
#define STEP_FM		250		/*   FM */
#define	STEP_CWZ	10		/*   CWZ ( )  */

static const uint8_t ssb_steps10 [] = 
{
	1,	/* 10 Hz */
	5,	/* 50 Hz */
	10,	/* 100 Hz */
};
static const uint8_t am_steps10 [] = 
{
	10,	/* 100 Hz */
	50,	/* 500 Hz */
	100,/* 1 kHz */
	150,/* 1.5 kHz */
	250,/* 2.5 kHz */
};

static uint16_t getstep(uint8_t submode)
{
	switch (submode)
	{
	default:
	case SUBMODE_USB: 
	case SUBMODE_LSB: 
	case SUBMODE_CW: 
	case SUBMODE_CWR: 
		//return g_STEP_SSB10 * 10U;
		return STEP_SSB;

	case SUBMODE_CWZ: 
		return STEP_CWZ;

	case SUBMODE_AM: 
		//return g_STEP_AM10 * 10U;
		return STEP_AM;

	case SUBMODE_DRM:
		return STEP_DRM;

	case SUBMODE_FM:
		return STEP_FM;
	}
}

/* ,     -    
     
   */

static uint16_t gstep;
static uint32_t dfreq;		/*      */
static uint8_t gsubmode;		/*    */
static uint8_t gfi;			/*   ()    */
static uint8_t att;
static uint8_t lockmode;
static uint8_t agcmode;
static uint16_t noaccelerate;	/*   . */
static uint8_t gmfixed;

static uint8_t moderow, modecol;

static uint16_t cwpitch10 = DEFAULT_CW_PITCH10;	/*     (  ) */

#if WITHMENU

struct menudef
{
	const char * name;
	uint8_t width, comma, rj;
	uint8_t step;
	uint16_t bottom, top;	/*    */
	uint16_t nvram;				/*   NVRAM */
	uint16_t * pval;			/* ,   */
};

static const struct menudef menutable [] =
{
	{
		"CW Pitch", 8, 2, 0, 	1, 40, 200, 
		offsetof(struct nvmap, cwpitch),
		& cwpitch10,
	},
	{
		"USB Edge", 8, 2, 0,	1,	44000, 46000, 
		offsetof(struct nvmap, usbe3p1),
		& fi_3p1.high10,
	},
	{
		"LSB Edge", 8, 2, 0,	1,	44000, 46000, 
		offsetof(struct nvmap, lsbe3p1),
		& fi_3p1.low10,
	},
	{
		"US6 Edge", 8, 2, 0,	1,	44000, 46000, 
		offsetof(struct nvmap, usbe6p0),
		& fi_6p0.high10,
	},
	{
		"LS6 Edge", 8, 2, 0,	1,	44000, 46000, 
		offsetof(struct nvmap, lsbe6p0),
		& fi_6p0.low10,

	},
	{
		"Enc Norm", 8, 2, 0,	1,	0, 2, 
		offsetof(struct nvmap, noaccelerate),
		& noaccelerate,
	},
};

#define MENUROW_COUNT (sizeof menutable / sizeof menutable [0])


/*    NVRAM   . 
       ,  
       .
   */
static void loadsettings(void)
{
	uint8_t i;

	for (i = 0; i < MENUROW_COUNT; ++ i)
	{
		* menutable [i].pval = 
			loadvfy16(menutable [i].nvram, 
			menutable [i].bottom, menutable [i].top, 
			* menutable [i].pval);
	}
}

void updateboard(unsigned char full);


/*     */
void modifysettings(void)
{
	unsigned char menupos;
	const struct menudef * mp;

	encoder_clear();

	menupos = loadvfy8(RMT_MENU_BASE, 0, MENUROW_COUNT, 0);
	mp = & menutable [menupos];
	lcd_menu_name(menupos, mp->name);
	lcd_menu_value(* mp->pval, mp->width, mp->comma, mp->rj);
	for (;;)
	{
		int nrotate;
		unsigned char kbch, repeat;

		ws1602_update();
		if (kbd_scan(& kbch, & repeat) != 0)
		{
			switch (kbch)
			{
			case KBD_CODE_B6 | KBD_LONGPRESS:
			case KBD_CODE_B6:
				/*    */
				save_i16(mp->nvram, * mp->pval);		/*    */
				encoder_clear();	//    
				return;

			case KBD_CODE_B7:
				/*      */
				save_i16(mp->nvram, * mp->pval);		/*    */
				menupos = calc_prev(menupos, 0, MENUROW_COUNT - 1);
				goto menuswitch;

			case KBD_CODE_B4:
				/*      */
				save_i16(mp->nvram, * mp->pval);		/*    */
				menupos = calc_next(menupos, 0, MENUROW_COUNT - 1);

			menuswitch:
				save_i8(RMT_MENU_BASE, menupos);
				mp = & menutable [menupos];
				
				lcd_clear();
				lcd_menu_name(menupos, mp->name);
				lcd_menu_value(* mp->pval, mp->width, mp->comma, mp->rj);
				break;
			}
		}

		/*      . */
		nrotate = getRotateLoRes();
		if (nrotate < 0)
		{
			* mp->pval =
				prevfreq(* mp->pval, * mp->pval - (- nrotate * mp->step), mp->step, mp->bottom);
			updateboard(1);
			lcd_menu_value(* mp->pval, mp->width, mp->comma, mp->rj);
		}
		else if (nrotate > 0)
		{
			* mp->pval =
				nextfreq(* mp->pval, * mp->pval + (nrotate * mp->step), mp->step, mp->top);
			updateboard(1);
			lcd_menu_value(* mp->pval, mp->width, mp->comma, mp->rj);
		}
		else
		{
		}
	}
}

#endif //WITHMENU


/*       
 */
static uint8_t getdetector(unsigned char submode)
{
	return gsubmodes [submode].detector;
}


static uint8_t getlsb(unsigned char submode)
{
	return gsubmodes [submode].lsb;
}

/*      
 */
static const filter_t * getfilter(unsigned char tx)
{
	return tx ? & fi_3p1 : gfa [gfi];
}


void updateboard(unsigned char full)
{
	/* ,   updateboard(1) */
	static unsigned long bfocalc;
	static unsigned long lo2;

	if (full != 0)
	{
		const filter_t * workfilter;	/*  ,     */
		uint32_t bfo;						/*    lo3 */

		/*        */
		workfilter = getfilter(0);
		board_set_att(att);
		board_set_agc(getagcoff(gsubmode, agcmode), getagccode(gsubmode, agcmode));
		board_set_filter(workfilter->code);
		board_set_detector(getdetector(gsubmode));
		board_set_lsb(getlsb(gsubmode));


		bfo = getlo3(workfilter, gsubmode, 0, cwpitch10, IF1_INVERSE);
		bfocalc = getlo3(workfilter, gsubmode, 1, cwpitch10, IF1_INVERSE);
		lo2 = getlo2(gsubmode);

		synth_lo3_setfreq(bfo, LO3_POWER);	/*    */
	}

	board_bandf_setfreq(dfreq);	/*     */
	if (IF1_INVERSE)
		synth_lo1_setfreq(dfreq + lo2 - bfocalc); /*     */
	else
		synth_lo1_setfreq(dfreq + lo2 + bfocalc); /*     */
}

/*   ,     
       ,  ((uint8_t)-1) */
static uint8_t 
getfreqband(uint32_t freq)
{
	const uint16_t f = (uint16_t) (freq / 1000);
	uint8_t i;

	for (i = 0; i < sizeof bandsmap / sizeof bandsmap [0]; ++ i)
	{
		if (bandsmap [i].bottom1k <= f && bandsmap [i].top1k > f)
			return i;
	}
	return ((uint8_t)-1);
}
/*       ,   .
    ,  low */
static uint8_t 
getnexthband(uint32_t freq, uint8_t low, uint8_t high)
{
	const uint16_t f = (uint16_t) (freq / 1000);
	uint8_t i;

	for (i = 0; i < HBANDS_COUNT; ++ i)
	{
		if (bandsmap [i].top1k > f)
			return i;
	}


	return low;
}

/*       ,   .
    ,  high */
static uint8_t 
getprevhband(uint32_t freq, uint8_t low, uint8_t high)
{
	const uint16_t f = (uint16_t) (freq / 1000);
	uint8_t i;

	for (i = 0; i < HBANDS_COUNT; ++ i)
	{
		if (bandsmap [i].bottom1k > f)
			return (i == 0) ? high : i - 1;
	}

	return high;
}


static uint8_t
getnextband(
	uint8_t b,
	uint8_t repeat
	)
{
	/*      ,
	    .
	   */
	uint32_t xfreq0 = loadvfy32freq(HBANDS_COUNT + 0);
	uint32_t xfreq1 = loadvfy32freq(HBANDS_COUNT + 1);

	uint8_t xnext0 = getnexthband(xfreq0, 0, HBANDS_COUNT - 1);
	uint8_t xprev0 = getprevhband(xfreq0, 0, HBANDS_COUNT - 1);

	uint8_t xnext1 = getnexthband(xfreq1, 0, HBANDS_COUNT - 1);
	uint8_t xprev1 = getprevhband(xfreq1, 0, HBANDS_COUNT - 1);

	while (repeat --)
	{
		if (b == xprev0)
		{
			b = HBANDS_COUNT + 0;
			continue;
		}
		if (b == xprev1)
		{
			b = HBANDS_COUNT + 1;
			continue;
		}
		if (b == HBANDS_COUNT + 0)
		{
			b = xnext0;
			continue;
		}
		if (b == HBANDS_COUNT + 1)
		{
			b = xnext1;
			continue;
		}
		if (b < HBANDS_COUNT)
		{
			b = calc_next(b, 0, HBANDS_COUNT - 1);
			continue;
		}
		else
		{
			b = getnexthband(dfreq, 0, HBANDS_COUNT - 1);
			continue;
		}
	}
	return b;
}


static uint8_t
getprevband(
	uint8_t b,
	uint8_t repeat
	)
{
	/*      ,
	    .
	   */
	uint32_t xfreq0 = loadvfy32freq(HBANDS_COUNT + 0);
	uint32_t xfreq1 = loadvfy32freq(HBANDS_COUNT + 1);

	uint8_t xnext0 = getnexthband(xfreq0, 0, HBANDS_COUNT - 1);
	uint8_t xprev0 = getprevhband(xfreq0, 0, HBANDS_COUNT - 1);

	uint8_t xnext1 = getnexthband(xfreq1, 0, HBANDS_COUNT - 1);
	uint8_t xprev1 = getprevhband(xfreq1, 0, HBANDS_COUNT - 1);

	while (repeat --)
	{
		if (b == xnext0)
		{
			b = HBANDS_COUNT + 0;
			continue;
		}
		if (b == xnext1)
		{
			b = HBANDS_COUNT + 1;
			continue;
		}
		if (b == HBANDS_COUNT + 0)
		{
			b = xprev0;
			continue;
		}
		if (b == HBANDS_COUNT + 1)
		{
			b = xprev1;
			continue;
		}
		if (b < HBANDS_COUNT)
		{
			b = calc_prev(b, 0, HBANDS_COUNT - 1);
			continue;
		}
		else
		{
			b = getprevhband(dfreq, 0, HBANDS_COUNT - 1);
		}
	}
	return b;
}


/*     -       */
static void savedfreq(void)
{
	save_i32(RMT_FREQ_BASE, dfreq);	/*      */
}


/*        */
static void
savebandstate(uint8_t b)
{
	save_i32(RMT_BFREQ_BASE(b), dfreq);	/*      */
	save_i8(RMT_MODEROW_BASE(b), moderow);
	save_i8(RMT_MODECOL_BASE(b) + moderow, modecol);
	save_i8(RMT_ATT_BASE(b), att);
}

/*  gsubmode      */
static void 
switchtonewband(uint8_t b)
{
	uint8_t defmode;		/*  -   */
	uint8_t defrow, defcol;

	att = loadvfy8(RMT_ATT_BASE(b), 0, 2, 0);	/*     */
	dfreq = loadvfy32freq(b);
	defmode = getdefaultmode(dfreq);		/*  -   */
	locatemode(defmode, & defrow, & defcol);	/* /  SSB */
	moderow = loadvfy8(RMT_MODEROW_BASE(b), 0, MODEROW_COUNT, defrow);
	modecol = loadvfy8(RMT_MODECOL_BASE(b) + moderow, 0, modes [moderow][0], defcol);	/* usage of defcol is wrong! */
	gsubmode = modes [moderow][modecol + 1];
	/*   */
	gfi = getsuitable(gsubmode, loadvfy8(RMT_FILTER_BASE(gsubmodes [gsubmode].mode), 0, sizeof gfa / sizeof gfa [0], 0));
	agcmode = loadvfy8(RMT_AGC_BASE(gsubmodes [gsubmode].mode), 0, AGCMODE_COUNT, AGCMODE_SLOW);
	gstep = getstep(gsubmode);
}

static void
setnewgmode(uint8_t gband, uint8_t newsubmode)
{
	int16_t delta10;
	uint8_t oldmode;
	oldmode = gsubmode;
	gsubmode = newsubmode;
	gstep = getstep(gsubmode);
	agcmode = loadvfy8(RMT_AGC_BASE(gsubmodes [gsubmode].mode), 0, AGCMODE_COUNT, AGCMODE_SLOW);
	gfi = getsuitable(gsubmode, loadvfy8(RMT_FILTER_BASE(gsubmodes [gsubmode].mode), 0, sizeof gfa / sizeof gfa [0], 0));
	/*             */
	delta10 = (gettone10(gsubmode, cwpitch10) - gettone10(oldmode, cwpitch10));

	/*    -  .
		   ,     ""  
		  -   ,      
		   ( )   .
		*/
	if (delta10 < 0)
	{
		/*   "" */
		dfreq = prevfreq(dfreq, dfreq - (- delta10) * 10UL, 10UL, TUNE_BOTTOM);
	}
	else if (delta10 > 0)
	{
		/*   "" */
		dfreq = nextfreq(dfreq, dfreq + delta10 * 10UL, 10UL, TUNE_TOP);
	}
	else
	{
	}
	savedfreq();
	savebandstate(gband);

}

/*   -  - , 
	      */
static uint8_t
processkeyboard(void)
{
	uint8_t gband;			/* ,      */
	uint8_t kbch, repeat;
	if (kbd_scan(& kbch, & repeat) != 0)
	{
		switch (kbch)
		{
		case KBD_CODE_B1:
			/*  
				 -      */
			while (repeat --)
				lockmode = calc_next(lockmode, 0, 1);
			save_i8(RMT_LOCKMODE_BASE, lockmode);
			return 1;

		case KBD_CODE_B1 | KBD_LONGPRESS:
			/*        . */
			while (repeat --)
				gmfixed = calc_next(gmfixed, 0, 1);
			save_i8(RMT_MFIXED_BASE, lockmode);
			return 1;


		case KBD_CODE_B5:
			/*  
				 -      */
			gfi = getsuitable(gsubmode, calc_next(gfi, 0, (sizeof gfa / sizeof gfa [0]) - 1));
			save_i8(RMT_FILTER_BASE(gsubmodes [gsubmode].mode), gfi);	/*        */
			return 1;

		case KBD_CODE_B6 | KBD_LONGPRESS:
			/*   
				 -      */
	#if WITHMENU
			gband = getfreqband(dfreq);	/*   ,     */
			lcd_clear();
			modifysettings();
			lcd_clear();
	#endif
			return 1;

		case KBD_CODE_B6:
			/*    - "F"
				 -      */
			return 1;

		case KBD_CODE_B2:
			/*   
				 -      */
			while (repeat --)
				att = calc_next(att, 0, 1);
			return 1;

		case KBD_CODE_B2 | KBD_LONGPRESS:
			/* AGC mode switch
				 -      */
			while (repeat --)
				agcmode = calc_next(agcmode, 0, AGCMODE_COUNT - 1);
			save_i8(RMT_AGC_BASE(gsubmodes [gsubmode].mode), agcmode);
			return 1;

		case KBD_CODE_B3 | KBD_LONGPRESS:
			/*   ""  -   */
			// step to next modecol
			gband = getfreqband(dfreq);	/*   ,     */
			while (repeat --)
				modecol = calc_next(modecol, 0, modes [moderow][0] - 1);
			/*        */
			/* gband     */
			setnewgmode(gband, modes [moderow][modecol + 1]);
			return 1;
	
		case KBD_CODE_B3:
			/*   ""  -   */
			/* switch to next moderow */
			{
				uint8_t defrow, defcol;

				gband = getfreqband(dfreq);	/*   ,     */
				while (repeat --)
					moderow = calc_next(moderow, 0, MODEROW_COUNT - 1);

				if (gsubmode == SUBMODE_USB)
					locatemode(SUBMODE_CW, & defrow, & defcol);	/* /  CW -   */
				else if (gsubmode == SUBMODE_LSB)
					locatemode(SUBMODE_CWR, & defrow, & defcol);	/* /  CW -   */
				if (defrow != moderow)
					modecol = 0;	/* default value (other cases, then switch from usb to cw, from lsb to cwr) */
				/* get modecol from NVRAM */
				modecol = loadvfy8(RMT_MODECOL_BASE(gband) + moderow, 0, modes [moderow][0], defcol);
				/*        */
				/* gband     */
				setnewgmode(gband, modes [moderow][modecol + 1]);
			}
			return 1;

		case KBD_CODE_B4:
			/*    (  )  */
			gband = getfreqband(dfreq);	/*   ,     */
			savebandstate(gband);
			gband = getnextband(gband, repeat);
			switchtonewband(gband);	/*      */
			savedfreq();
			return 1;

		case KBD_CODE_B7:
			/*    (  )  */
			gband = getfreqband(dfreq);	/*   ,     */
			savebandstate(gband);
			gband = getprevband(gband, repeat);
			switchtonewband(gband);	/*      */
			savedfreq();
			return 1;
		}
	}
	return 0;
}


/*    */
int main(void)
{
	uint8_t i;
	uint8_t dfreqmod;	/*     */
	uint8_t bclearnvram;

	//cpuio_init();		//     

	/*  -   power monitor    */
	for (i = 0; i < 50; ++ i)
	{
		_delay_ms(30);
	}

	board_init_io();		/*    SPI */
	encoder_initialize();
	timer_initialize();

	ws1602_initialize();
	kbd_initialize(& bclearnvram);

	nvram_initialize();	// write enable all memory regions
#if defined (NVRAM_TYPE)
	if (sizeof (struct nvmap) > NVRAM_SIZE)
	{
		ws1602_message("TOO LARGE nvmap");
		for (;;)
			;
	}
#endif

	if (bclearnvram)
	{
		/*    */
		uint16_t i;
		for (i = 0; i < NVRAM_SIZE; ++ i)
			save_i8(i, 0xFF);

	}

#if WITHMENU
	loadsettings();		/*     nvram */
#endif

	synth_init(SYNTH_DDSOSC, DDS_CLOCK_MULT);	//  .   

	board_bandf_initialize();

	sei();	// enable interrupts

	lockmode = loadvfy8(RMT_LOCKMODE_BASE, 0, 2, 0);	/*     */

	{
		uint8_t gband;
		dfreq = loadvfy32(RMT_FREQ_BASE, TUNE_BOTTOM, TUNE_TOP, 7040000Ul);

		/*  ""  nvram -       */
		gband = getfreqband(dfreq);	/*   ,     */
		savedfreq();	/*   */
		save_i32(RMT_BFREQ_BASE(gband), dfreq);	/*      */

		gmfixed = loadvfy8(RMT_MFIXED_BASE, 0, 2, 0);
		/*           */
		switchtonewband(gband);	/*      */
	}
	updateboard(1);	/*   (   ) */

	//
	noaccelerate = 0;

	/*   */
	lcd_clear();
	lcd_dispfreq(dfreq);
	lcd_dispmode(gsubmodes [gsubmode].name, noaccelerate, att, lockmode, agcmode2string [agcmode], getfilter(0)->name);

	encoder_clear();

	dfreqmod = 0;			/*     */
	for (;;)
	{
		uint32_t jumpsize;
		int16_t nrotate;

		ws1602_update();
		if (processkeyboard())
		{
			//   ,   ,   .  .
			//gstep = getstep(gsubmode);		/*         (  ). */
			/*       */
			dfreqmod = 0;
			encoder_clear();				/*        */
			updateboard(1);

			lcd_dispfreq(dfreq);
			lcd_dispmode(gsubmodes [gsubmode].name, noaccelerate, att, lockmode, agcmode2string [agcmode], getfilter(0)->name);
		} // end keyboard processing

		/*    -    gstep */
		nrotate = getRotateHiRes(& jumpsize, gstep);
	
		if (noaccelerate != 0)
			jumpsize = gstep;

		if (lockmode != 0)
			nrotate = 0;	// ignore encoder

		/*       */
		if (nrotate < 0)
		{
			/*  "" */
			dfreq = prevfreq(dfreq, dfreq - (jumpsize * - nrotate), gstep, TUNE_BOTTOM);
			dfreqmod = 1;	/*       -   */
			updateboard(0);	/*   -     */
			continue;
		}
		if (nrotate > 0)
		{
			/*  "" */
			dfreq = nextfreq(dfreq, dfreq + (jumpsize * nrotate), gstep, TUNE_TOP);
			dfreqmod = 1;	/*      -  */
			updateboard(0);	/*   -     */
			continue;
		}
		/*    -     -   */
		if (dfreqmod != 0)
		{

		/*      -    (  ). */
#if defined (NVRAM_TYPE) && (NVRAM_TYPE != NVRAM_TYPE_ATMEGA)
			savedfreq();	/*  ,    . */
#endif
			dfreqmod = 0;

			lcd_dispfreq(dfreq);
			continue;
		}
	}
	return 0;	
}
